"""
Notolog Editor
An open-source Markdown editor built with Python.

File Details:
- Purpose: Contains unit and integration tests for the related functionality.

Repository: https://github.com/notolog/notolog-editor
Website: https://notolog.app
PyPI: https://pypi.org/project/notolog

Author: Vadim Bakhrenkov
Copyright: 2024-2025 Vadim Bakhrenkov
License: MIT License

For detailed instructions and project information, please see the repository's README.md.
"""

from notolog.theme import Theme

from logging import Logger

import pytest
import os


class TestThemes:

    @pytest.fixture(scope="function", autouse=True)
    def test_obj_themes(self, mocker, request):
        """
        Testing object fixture.
        May contain minimal logic for testing purposes.
        """

        # Retrieve parameter values from the test request.
        theme = request.param if hasattr(request, 'param') else None

        """
        If themes dir is set read themes from there.
        For test purposes to check theme files are readable.
        """
        test_themes_dir = os.path.join(os.path.dirname(__file__), 'test_themes')
        mocker.patch.object(Theme, 'get_themes_dir', return_value=test_themes_dir)

        yield Theme(theme=theme)

    @pytest.fixture(scope="function")
    def test_exp_params_fixture(self, request):
        """
        Fixture to pass params and get expected results.
        This method is used only for passing params via pytest fixture.
        """

        # Retrieve parameter values from the test request.
        param_values = request.param

        yield param_values

    @pytest.mark.parametrize(
        "test_obj_themes, test_exp_params_fixture",
        [
            ((None), ('default')),
            ((''), ('default')),
            (('default'), ('default')),
            (('anyval'), ('anyval')),
        ],
        indirect=True
    )
    def test_themes_init(self, test_obj_themes, test_exp_params_fixture):
        """
        Test and check initial params are exist in the testing object.
        """
        assert isinstance(test_obj_themes.logger, Logger)

        exp_theme = test_exp_params_fixture

        assert hasattr(test_obj_themes, 'theme')
        assert test_obj_themes.theme == exp_theme
        assert hasattr(test_obj_themes, 'colors')
        assert hasattr(test_obj_themes, 'css')

    @pytest.mark.parametrize(
        "test_obj_themes, test_exp_params_fixture",
        [
            ((None), (None, 'default')),
            ((''), (None, 'default')),
            (('default'), (None, 'default')),
            (('anyval'), (None, 'anyval')),
            (('anyval'), ('new', 'new')),
        ],
        indirect=True
    )
    def test_themes_set_theme(self, test_obj_themes, test_exp_params_fixture):
        """
        Test theme setting for themes after the one was initialized.
        """
        param_theme, exp_theme = test_exp_params_fixture

        if param_theme:
            test_obj_themes.set_theme(param_theme)

        assert hasattr(test_obj_themes, 'theme')
        assert test_obj_themes.theme == exp_theme

    @pytest.mark.parametrize(
        "test_obj_themes, test_exp_params_fixture",
        [
            (None, os.path.join(os.path.dirname(__file__), 'test_themes')),
            ('default', os.path.join(os.path.dirname(__file__), 'test_themes')),
            ('noir_dark', os.path.join(os.path.dirname(__file__), 'test_themes')),
        ],
        indirect=True
    )
    def test_themes_get_themes_dir(self, test_obj_themes, test_exp_params_fixture):
        """
        Test theme dir path.
        """
        exp_themes_dir = test_exp_params_fixture

        assert test_obj_themes.get_themes_dir() == exp_themes_dir

    def test_themes_get_default_theme_dir(self, test_obj_themes):
        """
        Test default theme dir path.
        """

        # Use os.sep to ensure correct path separators on each OS, or os.path.normpath() to normalize path separators
        exp_themes_dir = 'test_themes' + os.sep + 'default'

        default_theme_dir_parts = test_obj_themes.get_default_theme_dir().split(os.path.sep)[-2:]

        assert os.path.join(*default_theme_dir_parts) == exp_themes_dir

    @pytest.mark.parametrize(
        "test_obj_themes, test_exp_params_fixture",
        [
            (None,
             ('default',
              {'default': {'test_red': '0xFF0000', 'test_green': '0x00FF00', 'test_blue': '0x0000FF',
                           'test_void': '0x000000'},
               'noir_dark': {'test_non_red': '0x00FFFF', 'test_non_green': '0xFF00FF', 'test_non_blue': '0xFFFF00',
                             'test_light': '0xFFFFFF'}})),
        ],
        indirect=True
    )
    def test_themes_load_themes(self, test_obj_themes, test_exp_params_fixture):
        """
        Test actual theme colors loaded.
        """
        exp_theme, exp_colors = test_exp_params_fixture

        assert hasattr(test_obj_themes, 'theme')
        assert test_obj_themes.theme == exp_theme
        assert hasattr(test_obj_themes, 'colors')

    @pytest.mark.parametrize(
        "test_obj_themes, test_exp_params_fixture",
        [
            (None,
             ('default', {'test_red': 0xFF0000, 'test_green': 0x00FF00, 'test_blue': 0x0000FF, 'test_void': 0x000000})),
            ('default',
             ('default',
              {'test_red': 0xFF0000, 'test_green': 0x00FF00, 'test_blue': 0x0000FF, 'test_void': 0x000000})),
            ('noir_dark',
             ('noir_dark',
              {'test_non_red': 0x00FFFF, 'test_non_green': 0xFF00FF, 'test_non_blue': 0xFFFF00,
               'test_light': 0xFFFFFF})),
        ],
        indirect=True
    )
    def test_themes_get_theme_colors(self, test_obj_themes, test_exp_params_fixture):
        """
        Test actual theme colors loaded.
        """
        exp_theme, exp_colors = test_exp_params_fixture

        assert hasattr(test_obj_themes, 'theme')
        assert test_obj_themes.theme == exp_theme
        assert test_obj_themes.get_colors() == exp_colors

    @pytest.mark.parametrize(
        "test_obj_themes, test_exp_params_fixture",
        [
            (None, ('default', {'test_styles': os.path.normpath('default/test_styles.css')})),
            ('default', ('default', {'test_styles': os.path.normpath('default/test_styles.css')})),
            ('noir_dark', ('noir_dark', {'test_smth': os.path.normpath('noir_dark/test_smth.css')})),
        ],
        indirect=True
    )
    def test_themes_get_theme_css(self, mocker, test_obj_themes, test_exp_params_fixture):
        """
        Test actual theme colors loaded.
        """
        exp_theme, exp_css = test_exp_params_fixture

        assert hasattr(test_obj_themes, 'theme')
        assert test_obj_themes.theme == exp_theme
        assert test_obj_themes.get_css().keys() == exp_css.keys()



# Chinese lexemes settings_dialog.py
lexemes = {
    # 设置
    "window_title": "设置",

    "button_close": "关闭",

    "tab_general": "通用",
    "tab_editor_config": "编辑器",
    "tab_viewer_config": "查看器",
    "tab_ai_config": "AI 配置",

    "general_app_config_label": "应用配置",
    "general_app_language_label": "语言",
    "general_app_language_combo_placeholder_text": "选择语言",
    "general_app_language_combo_accessible_description": "应用界面语言",
    "general_app_theme_label": "主题",
    "general_app_theme_combo_placeholder_text": "选择主题",
    "general_app_theme_combo_accessible_description": "应用界面主题",
    "general_app_main_menu_label": "主菜单",
    "general_app_main_menu_checkbox": "显示主菜单",
    "general_app_main_menu_checkbox_accessible_description": "显示应用的主下拉菜单",
    "general_app_font_size_label": "字体大小：{size}pt",
    "general_app_font_size_slider_accessible_description": "调整应用程序的全局字体大小",

    "general_statusbar_label": "状态栏",
    "general_statusbar_show_global_cursor_position_checkbox": "显示全局光标位置",
    "general_statusbar_show_global_cursor_position_checkbox_accessible_description":
        "在状态栏显示全局光标位置",

    "editor_config_label": "编辑器配置",
    "editor_config_show_line_numbers_checkbox": "显示行号",
    "editor_config_show_line_numbers_checkbox_accessible_description": "在编辑器中显示行号",

    "viewer_config_label": "查看器配置",
    "viewer_config_process_emojis_checkbox": "将文本表情转换为图形",
    "viewer_config_process_emojis_checkbox_accessible_description": "将文本表情转换为图形表示",
    "viewer_config_highlight_todos_checkbox": "突出显示待办事项",
    "viewer_config_highlight_todos_checkbox_accessible_description": "在文本中突出显示待办事项标签",
    "viewer_config_open_link_confirmation_checkbox": "打开链接前需确认",
    "viewer_config_open_link_confirmation_checkbox_accessible_description":
        "打开链接前询问确认",
    "viewer_config_save_resources_checkbox": "自动保存外部图片到硬盘",
    "viewer_config_save_resources_checkbox_accessible_description": "自动将外部图片的副本保存到硬盘以供离线访问。",

    "ai_config_inference_module_label": "推理模块",
    "ai_config_inference_module_names_combo_label": "活跃推理模块",
    "ai_config_inference_module_names_combo_placeholder_text": "选择模块",
    "ai_config_inference_module_names_combo_accessible_description":
        "从可用的AI推理模块中选择，以配合AI助手使用。\n"
        "选项包括具有实时处理功能的本地大型语言模型（LLM）或基于API的功能。",

    "ai_config_base_label": "基础参数",
    "ai_config_multi_turn_dialogue_checkbox": "带有会话记忆的多轮对话",
    "ai_config_multi_turn_dialogue_checkbox_accessible_description":
        "启用多轮对话，可保留上一次提示以进行会话记忆。\n"
        "关闭时，仅新消息和系统提示会影响回复。",
    "ai_config_convert_to_md_checkbox": "将结果转换为Markdown",
    "ai_config_convert_to_md_checkbox_accessible_description":
        "将输出消息转换为Markdown格式。",
}



# Hindi lexemes common.py
lexemes = {
    "app_title": "नोटोलॉग एडिटर",
    "app_title_with_sub": "{app_title} - {sub_title}",

    "tree_filter_accessible_desc": "फ़ाइल फ़िल्टर फ़ील्ड",

    "menu_action_copy_file_path": "फ़ाइल पथ की प्रतिलिपि बनाएँ",
    "menu_action_rename": "नाम बदलें",
    "menu_action_delete": "हटाएं",
    "menu_action_delete_completely": "पूरी तरह से हटाएं",
    "menu_action_restore": "पुनर्स्थापित करें",
    "menu_action_create_new_dir": "नई निर्देशिका बनाएँ",

    "dialog_file_rename_title": "फ़ाइल का नाम बदलें",
    "dialog_file_rename_field_label": "नया फ़ाइल नाम दर्ज करें",
    "dialog_file_rename_button_ok": "नाम बदलें",
    "dialog_file_rename_warning_exists": "इसी नाम की फ़ाइल पहले से मौजूद है",

    "dialog_file_delete_title": "फ़ाइल हटाएं",
    "dialog_file_delete_text": "क्या आप वाकई फ़ाइल \"{file_name}\" को हटाना चाहते हैं?",
    "dialog_file_delete_completely_title": "फ़ाइल को पूरी तरह से हटाएं",
    "dialog_file_delete_completely_text": "क्या आप वाकई फ़ाइल \"{file_name}\" को पूरी तरह से हटाना चाहते हैं?",
    "dialog_file_delete_error": "फ़ाइल हटाने में असमर्थ, त्रुटि हुई है",
    "dialog_file_delete_error_not_found": "फ़ाइल नहीं मिली",

    "dialog_file_restore_title": "फ़ाइल पुनर्स्थापित करें",
    "dialog_file_restore_text": "क्या आप वाकई फ़ाइल \"{file_name}\" को पुनर्स्थापित करना चाहते हैं?",
    "dialog_file_restore_error": "फ़ाइल को पुनर्स्थापित करने में असमर्थ, त्रुटि हुई है",
    "dialog_file_restore_warning_exists": "इस नाम की फ़ाइल पहले से मौजूद है",

    "dialog_create_new_dir_title": "नई निर्देशिका बनाएँ",
    "dialog_create_new_dir_label": "नई निर्देशिका का नाम",
    "dialog_create_new_dir_input_placeholder_text": "निर्देशिका नाम दर्ज करें",
    "dialog_create_new_dir_button_ok": "बनाएँ",
    "dialog_create_new_dir_button_cancel": "रद्द करें",
    "dialog_create_new_dir_warning_empty_title": "नई निर्देशिका नाम त्रुटि",
    "dialog_create_new_dir_warning_empty_text": "निर्देशिका नाम खाली नहीं हो सकता",
    "dialog_create_new_dir_warning_too_long_title": "नई निर्देशिका नाम त्रुटि",
    "dialog_create_new_dir_warning_too_long_text": "निर्देशिका का नाम बहुत लंबा है; अधिकतम "
                                                   "{symbols} वर्णों की अनुमति है!",
    "dialog_create_new_dir_error_existed": "निर्देशिका पहले से मौजूद है",
    "dialog_create_new_dir_error": "निर्देशिका नहीं बनाई जा सकती। सुनिश्चित करें कि गंतव्य निर्देशिका "
                                   "{base_dir} लिखने योग्य है",

    "dialog_message_box_title": "संदेश",
    "dialog_message_box_button_ok": "बंद करें",

    "action_new_file_first_line_template_text": "नया दस्तावेज़",
    "action_open_file_dialog_caption": "फ़ाइल खोलें",
    "action_save_as_file_dialog_caption": "फ़ाइल सहेजें",

    "dialog_save_empty_file_title": "खाली फ़ाइल सहेजें",
    "dialog_save_empty_file_text": "क्या खाली सामग्री के साथ फ़ाइल को सहेजने की अनुमति दें?",

    "dialog_encrypt_file_title": "फ़ाइल एन्क्रिप्ट करें",
    "dialog_encrypt_file_text": "क्या आप वाकई फ़ाइल \"{file_name}\" को एन्क्रिप्ट करना चाहते हैं?",
    "encrypt_file_warning_file_is_already_encrypted": "फ़ाइल पहले से ही एन्क्रिप्टेड है!",
    "dialog_encrypt_file_rewrite_existing_title": "मौजूदा फ़ाइल को पुनः लिखें",
    "dialog_encrypt_file_rewrite_existing_text": "क्या आप वाकई मौजूदा फ़ाइल \"{file_path}\" को पुनः लिखना चाहते हैं?",

    "dialog_decrypt_file_title": "फ़ाइल डिक्रिप्ट करें",
    "dialog_decrypt_file_text": "क्या आप वाकई फ़ाइल \"{file_name}\" को डिक्रिप्ट करना चाहते हैं?",
    "decrypt_file_warning_file_is_not_encrypted": "फ़ाइल एन्क्रिप्टेड नहीं है!",
    "dialog_decrypt_file_rewrite_existing_title": "मौजूदा फ़ाइल को पुनः लिखें",
    "dialog_decrypt_file_rewrite_existing_text": "क्या आप वाकई मौजूदा फ़ाइल \"{file_path}\" को पुनः लिखना चाहते हैं?",

    "dialog_encrypt_new_password_title": "नया पासवर्ड",
    "dialog_encrypt_new_password_label": "पासवर्ड:",
    "dialog_encrypt_new_password_input_placeholder_text": "नया पासवर्ड दर्ज करें",
    "dialog_encrypt_new_password_hint_label": "संकेत:",
    "dialog_encrypt_new_password_hint_label_description": "संकेत एन्क्रिप्टेड नहीं होता है और फ़ाइल से पढ़ा जा सकता है!"
                                                          "\nजन्म तिथियों जैसे आसानी से अनुमान लगाए जा सकने वाले "
                                                          "स्पष्ट संकेतों से बचें।\nआपके साथ आसानी से जुड़े नहीं जा "
                                                          "सकने वाले संदर्भ का प्रयोग करने का प्रयास करें।",
    "dialog_encrypt_new_password_hint_input_placeholder_text": "संकेत दर्ज करें (वैकल्पिक)",
    "dialog_encrypt_new_password_button_ok": "ठीक है",
    "dialog_encrypt_new_password_button_cancel": "रद्द करें",
    "dialog_encrypt_new_password_warning_empty_title": "चेतावनी",
    "dialog_encrypt_new_password_warning_empty_text": "पासवर्ड फ़ील्ड खाली नहीं हो सकती!",
    "dialog_encrypt_new_password_warning_too_long_title": "चेतावनी",
    "dialog_encrypt_new_password_warning_too_long_text": "संकेत फ़ील्ड बहुत लंबी है, अधिकतम {symbols} अक्षर!",

    "dialog_encrypt_password_title": "पासवर्ड दर्ज करें",
    "dialog_encrypt_password_label": "पासवर्ड:",
    "dialog_encrypt_password_input_placeholder_text": "पासवर्ड दर्ज करें",
    "dialog_encrypt_password_hint_label": "संकेत:",
    "dialog_encrypt_password_button_ok": "ठीक है",
    "dialog_encrypt_password_button_cancel": "रद्द करें",

    "dialog_encrypt_password_reset_title": "एन्क्रिप्शन पासवर्ड रीसेट करें",
    "dialog_encrypt_password_reset_text": "क्या आप वाकई मौजूदा एन्क्रिप्शन पासवर्ड को रीसेट करना चाहते हैं?",
    "dialog_encrypt_password_reset_button_cancel": "रद्द करें",
    "dialog_encrypt_password_reset_button_yes": "हाँ",

    "dialog_open_link_title": "लिंक",
    "dialog_open_link_text": "लिंक \"{url}\" को ब्राउज़र में खोलें?",

    "dialog_reset_settings_title": "सेटिंग्स रीसेट करें?",
    "dialog_reset_settings_text":
        "सेटिंग्स में संग्रहीत सभी डेटा मिटाया जाएगा, और एप्लिकेशन परिवर्तन लागू करने के लिए पुनः आरंभ होगा।",

    "dialog_exit_unsaved_title": "बाहर जाने की पुष्टि करें",
    "dialog_exit_unsaved_text": "खुली हुई फ़ाइल '{file_name}' को सहेजा नहीं जा सकता। क्या बाहर जाना जारी रखें?",

    "message_app_config_file_access": "{file_path} पर ऐप कॉन्फ़िगरेशन फ़ाइल तक पहुँचते समय अनुमति अस्वीकृत। सही "
                                      "ऑपरेशन सुनिश्चित करने के लिए सही अनुमतियाँ सेट करें।",

    "field_dir_path_dialog_caption": "डायरेक्टरी चुनें",
    "field_file_path_dialog_caption": "फ़ाइल चुनें",

    "load_file_encryption_password_mismatch": "एन्क्रिप्शन पासवर्ड मेल नहीं खाता!",
    "load_file_encryption_password_incorrect": "एन्क्रिप्शन पासवर्ड गलत है!",
    "load_file_none_content_error": "फ़ाइल लोड नहीं की जा सकती।",

    "action_new_file_error_occurred": "फ़ाइल नहीं बनाई जा सकती; एक त्रुटि हुई।\nफ़ाइल सिस्टम अनुमतियों की जाँच करें।",
    "save_active_file_error_occurred": "फ़ाइल सहेज नहीं सकते; एक त्रुटि हुई।",

    "expandable_block_default_title": "अधिक जानकारी...",
    "expandable_block_open_close_tags_mismatch_warning": "<details> ब्लॉक ओपन/क्लोज टैग मेल नहीं खाते",

    "dialog_color_picker_color_copied_to_the_clipboard": "स्वरूपित पाठ क्लिपबोर्ड में कॉपी किया गया है",

    "popup_about_title": "एप्लिकेशन की जानकारी",
    "popup_about_app_name_description": "पायथन मार्कडाउन एडिटर",

    "popup_about_version": "संस्करण",
    "popup_about_license": "लाइसेंस",
    "popup_about_website": "वेबसाइट",
    "popup_about_repository": "गिटहब",
    "popup_about_pypi": "पाइपी",
    "popup_about_date": "तारीख",

    "update_helper_new_version_is_available": "ऐप का नया संस्करण {latest_version} उपलब्ध है",
    "update_helper_latest_version_installed": "ऐप का नवीनतम संस्करण स्थापित है",

    "network_connection_error_empty": "प्रतिक्रिया जानकारी प्राप्त करने में असमर्थ",
    "network_connection_error_connection_or_dns": "होस्ट नहीं मिला। इंटरनेट कनेक्शन या DNS में समस्या हो सकती है।",
    "network_connection_error_connection_refused":
        "कनेक्शन अस्वीकृत किया गया। सर्वर बंद हो सकता है, या नेटवर्क में समस्या हो सकती है।",
    "network_connection_error_connection_timed_out": "कनेक्शन का समय समाप्त हो गया। नेटवर्क में समस्या हो सकती है।",
    "network_connection_error_connection_404_error": "कनेक्शन 404 त्रुटि। अनुरोधित पृष्ठ या संसाधन नहीं मिला।",
    "network_connection_error_generic_with_status_code": "स्थिति कोड के साथ अनुरोध विफल: {status_code}"
}



# Finnish lexemes common.py
lexemes = {
    # Common
    "module_ondevice_llm_name": "Laitteessa LLM",

    "module_ondevice_llm_model_exception": "Mallia ei löydy virhe: {error_msg}",
    "module_ondevice_llm_task_exception": "Malligeneraattoria ei voi alustaa: {error_msg}",
}



"""
Notolog Editor
An open-source Markdown editor built with Python.

File Details:
- Purpose: Part of the 'Module llama.cpp' module.

Repository: https://github.com/notolog/notolog-editor
Website: https://notolog.app
PyPI: https://pypi.org/project/notolog

Author: Vadim Bakhrenkov
Copyright: 2024-2025 Vadim Bakhrenkov
License: MIT License

For detailed instructions and project information, please see the repository's README.md.
"""

# This import is essential for proper module loading.
from .module_core import ModuleCore

from .. import AppConfig

import sys
import importlib.util


def get_name():
    """
    Readable module name.
    """
    return ModuleCore.module_name


def is_available() -> bool:
    # Required packages for a local LLM inference
    for package_name in ["llama_cpp"]:
        module_spec = importlib.util.find_spec(package_name)
        if module_spec is None:
            AppConfig().logger.info(f"The package '{package_name}' is required for the module '{get_name()}'")
            return False

    return True



"""
Notolog Editor
An open-source Markdown editor built with Python.

File Details:
- Purpose: Contains unit and integration tests for the related functionality.

Repository: https://github.com/notolog/notolog-editor
Website: https://notolog.app
PyPI: https://pypi.org/project/notolog

Author: Vadim Bakhrenkov
Copyright: 2024-2025 Vadim Bakhrenkov
License: MIT License

For detailed instructions and project information, please see the repository's README.md.
"""

from PySide6.QtCore import QCoreApplication

from notolog.app_config import AppConfig

import os
import pytest

# For headless operation
os.environ["QT_QPA_PLATFORM"] = "offscreen"


@pytest.fixture(autouse=True)
def test_core_app():

    # To correctly set up app settings
    QCoreApplication.setOrganizationName(AppConfig().get_settings_org_name())
    QCoreApplication.setApplicationName(AppConfig().get_settings_app_name_qa())

    AppConfig().set_test_mode(True)

    yield None



# Japanese lexemes common.py
lexemes = {
    "app_title": "Notolog エディタ",
    "app_title_with_sub": "{app_title} - {sub_title}",

    "tree_filter_accessible_desc": "ファイルフィルタフィールド",

    "menu_action_copy_file_path": "ファイルパスをコピー",
    "menu_action_rename": "名前を変更",
    "menu_action_delete": "削除",
    "menu_action_delete_completely": "完全に削除",
    "menu_action_restore": "復元",
    "menu_action_create_new_dir": "新しいディレクトリを作成",

    "dialog_file_rename_title": "ファイル名の変更",
    "dialog_file_rename_field_label": "新しいファイル名を入力",
    "dialog_file_rename_button_ok": "名前を変更",
    "dialog_file_rename_warning_exists": "同名のファイルがすでに存在しています",

    "dialog_file_delete_title": "ファイルの削除",
    "dialog_file_delete_text": "ファイル \"{file_name}\" を削除しますか？",
    "dialog_file_delete_completely_title": "ファイルを完全に削除",
    "dialog_file_delete_completely_text": "ファイル「{file_name}」を完全に削除しますか？",
    "dialog_file_delete_error": "ファイルを削除できませんでした。エラーが発生しました",
    "dialog_file_delete_error_not_found": "ファイルが見つかりません",

    "dialog_file_restore_title": "ファイルを復元",
    "dialog_file_restore_text": "ファイル \"{file_name}\" を復元しますか？",
    "dialog_file_restore_error": "ファイルを復元できませんでした。エラーが発生しました",
    "dialog_file_restore_warning_exists": "{file_name} という名前のファイルがすでに存在します",

    "dialog_create_new_dir_title": "新しいディレクトリを作成",
    "dialog_create_new_dir_label": "新しいディレクトリ名",
    "dialog_create_new_dir_input_placeholder_text": "ディレクトリ名を入力",
    "dialog_create_new_dir_button_ok": "作成",
    "dialog_create_new_dir_button_cancel": "キャンセル",
    "dialog_create_new_dir_warning_empty_title": "新しいディレクトリ名エラー",
    "dialog_create_new_dir_warning_empty_text": "ディレクトリ名を空にすることはできません",
    "dialog_create_new_dir_warning_too_long_title": "新しいディレクトリ名エラー",
    "dialog_create_new_dir_warning_too_long_text": "ディレクトリ名が長すぎます。最大 "
                                                   "{symbols} 文字が許可されています！",
    "dialog_create_new_dir_error_existed": "ディレクトリは既に存在します",
    "dialog_create_new_dir_error": "ディレクトリを作成できません。ターゲットディレクトリ "
                                   "{base_dir} が書き込み可能であることを確認してください",

    "dialog_message_box_title": "メッセージ",
    "dialog_message_box_button_ok": "閉じる",

    "action_new_file_first_line_template_text": "新しいドキュメント",
    "action_open_file_dialog_caption": "ファイルを開く",
    "action_save_as_file_dialog_caption": "ファイルを保存",

    "dialog_save_empty_file_title": "空のファイルを保存",
    "dialog_save_empty_file_text": "内容が空のファイルを保存しますか？",

    "dialog_encrypt_file_title": "ファイルの暗号化",
    "dialog_encrypt_file_text": "ファイル \"{file_name}\" を暗号化しますか？",
    "encrypt_file_warning_file_is_already_encrypted": "ファイルはすでに暗号化されています！",
    "dialog_encrypt_file_rewrite_existing_title": "既存のファイルを上書き",
    "dialog_encrypt_file_rewrite_existing_text": "既存のファイル \"{file_path}\" を上書きしますか？",

    "dialog_decrypt_file_title": "ファイルの復号",
    "dialog_decrypt_file_text": "ファイル \"{file_name}\" を復号しますか？",
    "decrypt_file_warning_file_is_not_encrypted": "ファイルは暗号化されていません！",
    "dialog_decrypt_file_rewrite_existing_title": "既存のファイルを上書き",
    "dialog_decrypt_file_rewrite_existing_text": "既存のファイル \"{file_path}\" を上書きしますか？",

    "dialog_encrypt_new_password_title": "新しいパスワード",
    "dialog_encrypt_new_password_label": "パスワード：",
    "dialog_encrypt_new_password_input_placeholder_text": "新しいパスワードを入力",
    "dialog_encrypt_new_password_hint_label": "ヒント：",
    "dialog_encrypt_new_password_hint_label_description": "ヒントは暗号化されずにファイルから読み取ることができます！"
                                                          "\n誕生日など簡単に推測できるヒントは使用しないでください。"
                                                          "\n関連性の低い参照を使用してください。",
    "dialog_encrypt_new_password_hint_input_placeholder_text": "ヒントを入力（オプション）",
    "dialog_encrypt_new_password_button_ok": "OK",
    "dialog_encrypt_new_password_button_cancel": "キャンセル",
    "dialog_encrypt_new_password_warning_empty_title": "警告",
    "dialog_encrypt_new_password_warning_empty_text": "パスワード欄は空にできません！",
    "dialog_encrypt_new_password_warning_too_long_title": "警告",
    "dialog_encrypt_new_password_warning_too_long_text": "ヒント欄が長すぎます。最大{symbols}文字です！",

    "dialog_encrypt_password_title": "パスワードを入力",
    "dialog_encrypt_password_label": "パスワード：",
    "dialog_encrypt_password_input_placeholder_text": "パスワードを入力",
    "dialog_encrypt_password_hint_label": "ヒント：",
    "dialog_encrypt_password_button_ok": "OK",
    "dialog_encrypt_password_button_cancel": "キャンセル",

    "dialog_encrypt_password_reset_title": "暗号化パスワードのリセット",
    "dialog_encrypt_password_reset_text": "現在の暗号化パスワードをリセットしますか？",
    "dialog_encrypt_password_reset_button_cancel": "キャンセル",
    "dialog_encrypt_password_reset_button_yes": "はい",

    "dialog_open_link_title": "リンク",
    "dialog_open_link_text": "リンク \"{url}\" をブラウザで開きますか？",

    "dialog_reset_settings_title": "設定をリセットしますか？",
    "dialog_reset_settings_text": "設定に保存されているすべてのデータがクリアされ、変更を適用するためにアプリが再起動されます。",

    "dialog_exit_unsaved_title": "終了の確認",
    "dialog_exit_unsaved_text": "開いているファイル '{file_name}' は保存できません。終了しますか？",

    "message_app_config_file_access": "{file_path} でアプリ設定ファイルにアクセスする際に許可が拒否されました。"
                                      "適切な操作を確保するために正しい許可を設定してください。",

    "field_dir_path_dialog_caption": "ディレクトリを選択",
    "field_file_path_dialog_caption": "ファイルを選択",

    "load_file_encryption_password_mismatch": "暗号化パスワードが一致しません！",
    "load_file_encryption_password_incorrect": "暗号化パスワードが正しくありません！",
    "load_file_none_content_error": "ファイルを読み込めません。",

    "action_new_file_error_occurred": "ファイルを作成できませんでした；エラーが発生しました。\nファイルシステムの権限を確認してください。",
    "save_active_file_error_occurred": "ファイルを保存できませんでした；エラーが発生しました。",

    "expandable_block_default_title": "詳細情報...",
    "expandable_block_open_close_tags_mismatch_warning": "<details> ブロックの開始/終了タグの不一致",

    "dialog_color_picker_color_copied_to_the_clipboard": "フォーマットされたテキストがクリップボードにコピーされました",

    "popup_about_title": "アプリ情報",
    "popup_about_app_name_description": "Python Markdown エディタ",

    "popup_about_version": "バージョン",
    "popup_about_license": "ライセンス",
    "popup_about_website": "ウェブサイト",
    "popup_about_repository": "GitHub",
    "popup_about_pypi": "PyPi",
    "popup_about_date": "日付",

    "update_helper_new_version_is_available": "アプリの新しいバージョン {latest_version} が利用可能です",
    "update_helper_latest_version_installed": "アプリの最新バージョンがインストールされています",

    "network_connection_error_empty": "応答情報を取得できません",
    "network_connection_error_connection_or_dns":
        "ホストが見つかりません。インターネット接続またはDNSに問題があるかもしれません。",
    "network_connection_error_connection_refused":
        "接続が拒否されました。サーバーがダウンしているか、ネットワークに問題がある可能性があります。",
    "network_connection_error_connection_timed_out": "接続がタイムアウトしました。ネットワークに問題があるかもしれません。",
    "network_connection_error_connection_404_error":
        "接続404エラー。要求されたページまたはリソースが見つかりません。",
    "network_connection_error_generic_with_status_code": "ステータスコード{status_code}でリクエストが失敗しました",
}



"""
Notolog Editor
An open-source Markdown editor built with Python.

File Details:
- Purpose: Contains unit and integration tests for the related functionality.

Repository: https://github.com/notolog/notolog-editor
Website: https://notolog.app
PyPI: https://pypi.org/project/notolog

Author: Vadim Bakhrenkov
Copyright: 2024-2025 Vadim Bakhrenkov
License: MIT License

For detailed instructions and project information, please see the repository's README.md.
"""

from notolog.lexemes.lexemes import Lexemes

from logging import Logger

import os
import pytest
import pathlib
import importlib.util


class TestLexemes:

    @pytest.fixture(scope="function")
    def test_obj_lexemes(self, mocker, request):
        """
        Testing object fixture.
        May contain minimal logic for testing purposes.
        """

        # Retrieve parameter values from the test request.
        language, lexemes, lexemes_dir = request.param

        if lexemes:
            """
            If lexemes explicitly set return them.
            This is for test cases where the lexemes file read is not necessary.
            """
            mocker.patch.object(Lexemes, 'load_lexemes', return_value=lexemes)
        elif lexemes_dir:
            """
            If lexemes dir is set read lexemes from there.
            For test cases to check actual lexemes file read.
            """
            test_lexemes_dir = os.path.join(os.path.dirname(__file__), lexemes_dir)
            mocker.patch.object(Lexemes, 'get_lexemes_dir', return_value=test_lexemes_dir)
        else:
            """
            Return empty lexemes in the rest of the cases.
            """
            mocker.patch.object(Lexemes, 'load_lexemes', return_value={})

        yield Lexemes(language=language)

    @pytest.fixture(scope="function")
    def test_exp_params_fixture(self, request):
        """
        Fixture to pass params and get expected results.
        This method is used only for passing params via pytest fixture.
        """

        # Retrieve parameter values from the test request.
        param_values = request.param

        yield param_values

    @pytest.mark.parametrize(
        "test_obj_lexemes, test_exp_params_fixture",
        [
            ((None, None, None), ('en', {})),
            (('', {}, None), ('en', {})),
            (('en', {123: 456}, None), ('en', {123: 456})),
            (('anyval', {'A': 'B', 'C': 'D'}, None), ('anyval', {'A': 'B', 'C': 'D'})),
        ],
        indirect=True
    )
    def test_lexemes_init(self, test_obj_lexemes: Lexemes, test_exp_params_fixture):
        """
        Test and check initial params are exist in the testing object.
        """
        assert isinstance(test_obj_lexemes.logger, Logger)

        exp_language, exp_lexemes = test_exp_params_fixture

        assert hasattr(test_obj_lexemes, 'language')
        assert test_obj_lexemes.language == exp_language
        assert hasattr(test_obj_lexemes, 'lexemes')
        assert test_obj_lexemes.lexemes == exp_lexemes

    @pytest.mark.parametrize(
        "test_obj_lexemes, test_exp_params_fixture",
        [
            ((None, None, None), (None, 'en')),
            (('', None, None), (None, 'en')),
            (('en', None, None), (None, 'en')),
            (('anyval', None, None), (None, 'anyval')),
            (('anyval', None, None), ('new', 'new')),
        ],
        indirect=True
    )
    def test_lexemes_set_language(self, test_obj_lexemes: Lexemes, test_exp_params_fixture):
        """
        Test language setting for lexemes after the one was initialized.
        """
        param_language, exp_language = test_exp_params_fixture

        if param_language:
            test_obj_lexemes.set_language(param_language)

        assert hasattr(test_obj_lexemes, 'language')
        assert test_obj_lexemes.language == exp_language

    @pytest.mark.parametrize(
        "test_obj_lexemes, test_exp_params_fixture",
        [
            ((None, None, None), 'en'),
            (('', None, None), 'en'),
            (('en', None, None), 'en'),
            (('any', None, None), 'en'),
            (('..', None, None), 'en'),
        ],
        indirect=True
    )
    def test_lexemes_get_lexemes_dir(self, mocker, test_obj_lexemes: Lexemes, test_exp_params_fixture):
        """
        Test lexemes target dir path.
        """
        exp_lexemes_dir = test_exp_params_fixture

        mocker.patch.object(os.path, 'dirname', return_value='')

        assert test_obj_lexemes.get_lexemes_dir() == exp_lexemes_dir

    @pytest.mark.parametrize(
        "test_obj_lexemes, test_exp_params_fixture",
        [
            ((None, None, None),
             ('en', {})),
            ((None, None, 'test_lexemes'),
             ('en', {'test_common': {'any': 'thing', 'hello': 'Hello World!', 'test': 'Passed'},
                     'test_scope': {123: 456, 'foo': 'bar', 'test': 'result'}})),
        ],
        indirect=True
    )
    def test_lexemes_load_lexemes(self, test_obj_lexemes: Lexemes, test_exp_params_fixture):
        """
        Test actual lexemes loaded.
        """
        exp_language, exp_lexemes = test_exp_params_fixture

        assert hasattr(test_obj_lexemes, 'lexemes')
        assert test_obj_lexemes.lexemes == exp_lexemes

    @pytest.mark.parametrize(
        "test_obj_lexemes, test_exp_params_fixture",
        [
            ((None, None, None), (None, None, None)),
            ((None, None, 'test_lexemes'), ('test_common', 'hello', 'Hello World!')),
            ((None, None, 'test_lexemes'), ('test_common', 'test', 'Passed')),
            ((None, None, 'test_lexemes'), ('test_scope', 123, 456)),
            ((None, None, 'test_lexemes'), ('test_scope', 'undefined_name', '')),
            ((None, None, 'test_lexemes'), ('test_scope_undefined', None, None)),
            ((None, None, 'test_lexemes'), ('test_scope_undefined', 'undefined_name', None)),
        ],
        indirect=True
    )
    def test_lexemes_get_all(self, test_obj_lexemes: Lexemes, test_exp_params_fixture):
        """
        Test getter for correctly loaded and really existent lexemes.
        """
        param_scope, param_name, exp_result = test_exp_params_fixture

        _all_lexemes = test_obj_lexemes.get_all()

        if param_scope:
            assert len(_all_lexemes) > 0
            _scope_lexemes = _all_lexemes[param_scope] if param_scope in _all_lexemes else None
            if _scope_lexemes:
                if param_name in _scope_lexemes:
                    assert _scope_lexemes[param_name] == exp_result
                else:
                    assert exp_result == ''
            else:
                assert exp_result is None

    @pytest.mark.parametrize(
        "test_obj_lexemes, test_exp_params_fixture",
        [
            ((None, None, None), (None, None, None)),
            ((None, None, 'test_lexemes'), ('test_common', 'hello', 'Hello World!')),
            ((None, None, 'test_lexemes'), ('test_common', 'test', 'Passed')),
            ((None, None, 'test_lexemes'), ('test_scope', 123, 456)),
            ((None, None, 'test_lexemes'), ('test_scope', 'undefined_name', '')),
            ((None, None, 'test_lexemes'), ('test_scope_undefined', None, None)),
            ((None, None, 'test_lexemes'), ('test_scope_undefined', 'undefined_name', None)),
        ],
        indirect=True
    )
    def test_lexemes_get_by_scope(self, test_obj_lexemes: Lexemes, test_exp_params_fixture):
        """
        Test getter for correctly loaded and really existent lexemes.
        """
        param_scope, param_name, exp_result = test_exp_params_fixture

        _scope_lexemes = test_obj_lexemes.get_by_scope(param_scope)
        if not _scope_lexemes:
            assert exp_result is None
        else:
            if param_name in _scope_lexemes:
                assert _scope_lexemes[param_name] == exp_result
            else:
                assert exp_result == ''

    @pytest.mark.parametrize(
        "test_obj_lexemes, test_exp_params_fixture",
        [
            ((None, None, None), (None, None, None)),
            ((None, None, 'test_lexemes'), ('test_common', 'hello', 'Hello World!')),
            ((None, None, 'test_lexemes'), ('test_common', 'test', 'Passed')),
            ((None, None, 'test_lexemes'), ('test_scope', 123, 456)),
            ((None, None, 'test_lexemes'), ('test_scope', 'undefined_name', '')),
            ((None, None, 'test_lexemes'), ('test_scope_undefined', None, None)),
            ((None, None, 'test_lexemes'), ('test_scope_undefined', 'undefined_name', None)),
        ],
        indirect=True
    )
    def test_lexemes_get(self, test_obj_lexemes: Lexemes, test_exp_params_fixture):
        """
        Test getter for correctly loaded and really existent lexemes.
        """
        param_scope, param_name, exp_result = test_exp_params_fixture

        assert test_obj_lexemes.get(param_name, param_scope) == exp_result

    @staticmethod
    def load_module_from_file(full_path_to_module, module_name):
        spec = importlib.util.spec_from_file_location(module_name, full_path_to_module)
        module = importlib.util.module_from_spec(spec)
        spec.loader.exec_module(module)
        return module

    def test_translation_files(self):
        # Define the root path for translation directories
        translations_root = os.path.join(
            pathlib.Path(os.path.dirname(__file__)).parent.resolve(), 'notolog', 'lexemes')

        # Iterate over each language directory
        for lang in os.listdir(translations_root):
            lang_path = os.path.join(translations_root, lang)
            if os.path.isdir(lang_path):
                # Iterate over each module in the language directory
                for file in os.listdir(lang_path):
                    if file.endswith('.py'):
                        # Load the module
                        module_path = os.path.join(lang_path, file)
                        module_name = f"{lang}.{file[:-3]}"  # Remove '.py' from filename, to get smth like 'toolbar'
                        module = self.load_module_from_file(module_path, module_name)

                        # You can now use 'module' to access the loaded module's attributes
                        # and ensure they are covered by some form of assertion or operation.
                        # For this example, let's just print out the module's dir(): print(dir(module))
                        assert module_name == (f"{os.path.basename(lang_path)}."
                                               f"{os.path.basename(file).replace('.py', '')}")

                        # Check the actual language data exists
                        assert isinstance(module.lexemes, dict)



# Portuguese lexemes main_menu.py
lexemes = {
    "group_file_label": "Arquivo",
    "group_edit_label": "Editar",
    "group_tools_label": "Ferramentas",
    "group_help_label": "Ajuda",

    "actions_file_label_new_document": "Novo documento",
    "actions_file_accessible_name_new_document": "Novo documento",
    "actions_file_label_open": "Abrir",
    "actions_file_accessible_name_open": "Abrir documento",
    "actions_file_label_save": "Salvar",
    "actions_file_accessible_name_save": "Salvar documento",
    "actions_file_label_save_as": "Salvar como",
    "actions_file_accessible_name_save_as": "Salvar como cópia do documento",
    "actions_file_label_settings": "Configurações",
    "actions_file_accessible_name_settings": "Configurações do aplicativo",
    "actions_file_label_reset_settings": "Redefinir configurações",
    "actions_file_accessible_name_reset_settings": "Redefinir configurações",
    "actions_file_label_exit": "Sair do aplicativo",
    "actions_file_accessible_name_exit": "Sair do aplicativo",

    "actions_edit_edit_mode": "Editar",
    "actions_edit_accessible_name_edit_mode": "Editar arquivo",
    "actions_edit_source_mode": "Fonte",
    "actions_edit_accessible_name_source_mode": "Código-fonte",
    "actions_edit_bold": "Negrito",
    "actions_edit_accessible_name_bold": "Formato de texto em negrito",
    "actions_edit_italic": "Itálico",
    "actions_edit_accessible_name_italic": "Formato de texto em itálico",
    "actions_edit_underline": "Sublinhado",
    "actions_edit_accessible_name_underline": "Formato de texto sublinhado",
    "actions_edit_strikethrough": "Riscado",
    "actions_edit_accessible_name_strikethrough": "Formato de texto riscado",
    "actions_edit_blockquote": "Citação",
    "actions_edit_accessible_name_blockquote": "Formato de texto de citação",

    "actions_tools_ai_assistant": "Assistente de IA",
    "actions_tools_accessible_name_ai_assistant": "Lançar Assistente de IA",
    "actions_tools_color_picker": "Seletor de Cor",
    "actions_tools_accessible_name_color_picker": "Escolher uma cor para o texto",

    "actions_help_label_md_syntax": "Sintaxe Markdown",
    "actions_help_accessible_name_md_syntax": "Sintaxe Markdown",
    "actions_help_label_check_for_updates": "Verificar atualizações",
    "actions_help_accessible_name_check_for_updates": "Verificar atualizações",
    "actions_help_label_bug_report": "Relatar bug",
    "actions_help_accessible_name_bug_report": "Enviar relatório de bug",
    "actions_help_label_about": "Sobre",
    "actions_help_accessible_name_about": "Sobre",
}



# Japanese lexemes ai_assistant.py
lexemes = {
    # AI アシスタント
    "dialog_title": "AI へのリクエスト",
    "dialog_prompt_input_placeholder_text": "リクエスト",
    "dialog_prompt_input_accessible_description": "AIアシスタントにコンテキストを提供するためのリクエスト入力",
    "dialog_response_output_placeholder_text": "応答データ",
    "dialog_response_output_accessible_description": "応答データはこのフィールドに表示されます",
    "dialog_response_output_notice_empty_text": "テキスト入力を提供して、約...",

    "dialog_usage_module_label": "モジュール",
    "dialog_usage_model_label": "モデル",
    "dialog_usage_tokens_label": "トークン",
    "dialog_usage_tokens_prompt": "プロンプト: {tokens}",
    "dialog_usage_tokens_answer": "回答: {tokens}",
    "dialog_usage_tokens_total": "合計: {tokens}",

    "dialog_button_send_request": "リクエストを送信",
    "dialog_button_stop_request": "停止",
    "dialog_button_save_history": "対話履歴を保存する",

    "dialog_message_copy_tooltip": "テキストをコピー",
    "dialog_message_copied_tooltip": "コピーしました",

    "dialog_error_loading_model": "カスタムモデルのロードエラー",
}



# Japanese lexemes common.py
lexemes = {
    "module_llama_cpp_name": "モジュール llama.cpp",

    "module_llama_cpp_model_exception": "モデルが見つからないエラー: {error_msg}",
    "module_llama_cpp_task_exception": "モデルジェネレータを初期化できません: {error_msg}",
}



# Greek lexemes ai_assistant.py
lexemes = {
    # Διάλογος βοηθού AI
    "dialog_title": "Αίτηση προς το AI",
    "dialog_prompt_input_placeholder_text": "Αίτηση",
    "dialog_prompt_input_accessible_description": "Εισαγωγή αιτήματος για να δώσετε πλαίσιο στον βοηθό AI",
    "dialog_response_output_placeholder_text": "Δεδομένα απόκρισης",
    "dialog_response_output_accessible_description": "Τα δεδομένα απόκρισης θα εμφανιστούν σε αυτό το πεδίο",
    "dialog_response_output_notice_empty_text": "Παρέχετε εισαγωγή κειμένου για να λάβετε μια απάντηση περίπου...",

    "dialog_usage_module_label": "ΜΟΔΟΥΛΟ",
    "dialog_usage_model_label": "ΜΟΝΤΕΛΟ",
    "dialog_usage_tokens_label": "ΔΙΑΚΡΙΤΙΚΑ",
    "dialog_usage_tokens_prompt": "prompt: {tokens}",
    "dialog_usage_tokens_answer": "answer: {tokens}",
    "dialog_usage_tokens_total": "total: {tokens}",

    "dialog_button_send_request": "Αποστολή Αιτήματος",
    "dialog_button_stop_request": "Διακοπή",
    "dialog_button_save_history": "Αποθήκευση ιστορικού διαλόγου",

    "dialog_message_copy_tooltip": "Αντιγραφή κειμένου",
    "dialog_message_copied_tooltip": "Αντιγράφηκε",

    "dialog_error_loading_model": "Σφάλμα φόρτωσης προσαρμοσμένου μοντέλου",
}



"""
Notolog Editor
An open-source Markdown editor built with Python.

File Details:
- Purpose: Settings functionality.
- Functionality: Allows adding new setting elements via the config map and handling settings change signals.

Repository: https://github.com/notolog/notolog-editor
Website: https://notolog.app
PyPI: https://pypi.org/project/notolog

Author: Vadim Bakhrenkov
Copyright: 2024-2025 Vadim Bakhrenkov
License: MIT License

For detailed instructions and project information, please see the repository's README.md.
"""

from PySide6.QtCore import Signal, QSettings

from .app_config import AppConfig
from .enums.themes import Themes
from .enums.languages import Languages
from .helpers.settings_helper import SettingsHelper
from .helpers.file_helper import is_writable_path

from typing import TYPE_CHECKING, Any
from threading import Lock

import os
import logging

if TYPE_CHECKING:
    from typing import Union  # noqa: F401


class Settings(QSettings):
    """
    Class to store app settings params.
    """

    value_changed = Signal(object)  # type: Signal[Union[object, QSettings]]

    _instance = None  # Singleton instance
    _lock = Lock()

    ai_config_inference_modules: dict = {}

    def __new__(cls, *args, **kwargs):
        if not cls._instance:
            with cls._lock:
                if not cls._instance:
                    # Create the instance if it doesn't exist
                    cls._instance = super().__new__(cls)
        return cls._instance

    @classmethod
    def reload(cls, *args, **kwargs):
        """
        Reinitialize the singleton instance. This method allows for the controlled
        re-creation of the singleton instance.
        """
        with cls._lock:
            # Create a new instance
            cls._instance = super().__new__(cls)

    def __init__(self, parent=None):
        """
        Args:
            parent (optional): Parent object
        """

        # Prevent re-initialization if the instance is already set up.
        if hasattr(self, 'settings'):
            return

        # Use a lock to ensure initialization is thread-safe and atomic.
        with self._lock:
            # Double-check to prevent race conditions during initialization.
            if hasattr(self, 'settings'):
                return

            super(Settings, self).__init__(parent)

            self.logger = logging.getLogger('settings')

            self.settings = self.get_instance()
            self.settings_helper = SettingsHelper()

            # Attributes may not be set at the very beginning
            self.logger.debug('Window size %d x %d' % (getattr(self, 'ui_width', 0), getattr(self, 'ui_height', 0)))

            self.value_changed.connect(lambda v: self.settings.sync())

            self.protected_attr = []
            self.init_fields()

    def get_instance(self):
        settings_file_path = self.get_filename()
        # Check file permissions
        if not is_writable_path(settings_file_path):
            self.logger.warning(f"Permission denied: Cannot write to the file {settings_file_path}")
            settings_file_path = None
        # Initialize settings with either a custom or the default path
        if settings_file_path:
            return QSettings(settings_file_path, QSettings.Format.NativeFormat)
        else:
            return QSettings()

    def get_filename(self):
        # Get app's package
        app_package = AppConfig().get_package_type()
        # Get default path based on the application config
        settings_file_path = self.fileName()
        # Adjust path if the specific package is in use
        if app_package != AppConfig.default_package:
            # Split the file path into directory, filename without extension, and extension
            directory, file = os.path.split(settings_file_path)
            _file_name, _file_ext = os.path.splitext(file)
            settings_file_path = os.path.join(directory, f'{_file_name}_{app_package}{_file_ext}')
        return settings_file_path

    def init_fields(self):
        # App's UI settings
        self.create_property("app_theme", str, str(Themes.default()))
        self.create_property("app_language", str, str(Languages.default()))
        self.create_property("app_font_size", int, AppConfig().get_font_base_size(),
                             set_prev_func=lambda prev_value: AppConfig().set_prev_font_size(prev_value))
        # Window positioning and size
        self.create_property("ui_width", int, 0)
        self.create_property("ui_height", int, 0)
        self.create_property("ui_pos_x", int, 0)
        self.create_property("ui_pos_y", int, 0)
        # Splitter between navigation tree and view/edit widget position
        self.create_property("ui_splitter_pos", int, 0)
        # Global settings
        self.create_property("show_main_menu", bool, True)
        self.create_property("show_deleted_files", bool, False)
        self.create_property("show_global_cursor_position", bool, False)
        # Editor state
        self.create_property("file_path", str, "")
        self.create_property("mode", int, 0)
        self.create_property("source", int, 0)
        self.create_property("line_num", int, 0)
        self.create_property("col_num", int, 0)
        self.create_property("cursor_pos", int, 0)
        self.create_property("viewport_pos", list, [0, 0])  # Scroll doc to this position
        self.create_property("show_line_numbers", bool, True)
        # Viewer state
        self.create_property("viewer_process_emojis", bool, True)
        self.create_property("viewer_highlight_todos", bool, True)
        self.create_property("viewer_open_link_confirmation", bool, True)
        self.create_property("viewer_save_resources", bool, True)
        # Icons to show on the toolbar. Can be checked like:
        # if value is not None and value.isValid() and isinstance(value.toInt()[0], int): ...
        self.create_property("toolbar_icons", int, None)
        self.create_property("enc_iterations", int, None)
        # AI assistant
        self.create_property("ai_config_inference_module", str, "openai_api")
        self.create_property("ai_config_multi_turn_dialogue", bool, True)
        self.create_property("ai_config_convert_to_md", bool, False)

    def create_property(self, param_name: str, param_type: Any, default_value: Any, set_prev_func: Any = None,
                        encrypt: bool = False):
        def getter(_self):
            # Type is crucial here
            value = _self.settings.value(param_name, default_value, type=param_type)
            if encrypt:
                try:
                    value = self.settings_helper.decrypt_data(value)
                except (ValueError, EnvironmentError) as e:
                    self.logger.warning(f'Settings protected parameter "{param_name}" getting error: {e}')
            return param_type(value)

        def setter(_self, value):
            _prev_value = _self.settings.value(param_name)
            if _prev_value != value:
                if callable(set_prev_func):
                    # Save previous value
                    set_prev_func(_prev_value)
                if encrypt:
                    try:
                        value = self.settings_helper.encrypt_data(value)
                    except (ValueError, EnvironmentError) as e:
                        self.logger.warning(f'Settings protected parameter "{param_name}" setting error: {e}')
                _self.settings.setValue(param_name, value)
                _self.value_changed.emit({param_name: value})

        # Dynamically create the property and its setter
        setattr(Settings, param_name, property(getter, setter))

        # Add param to the protected list if applicable
        if encrypt:
            self.protected_attr.append(param_name)

    """
    Example of how to set up for each settings property
    """

    @property
    def show_main_menu(self) -> bool:
        # From settings UI dialog
        return self.settings.value("show_main_menu", True, type=bool)  # type: ignore

    @show_main_menu.setter
    def show_main_menu(self, value) -> None:
        self.settings.setValue("show_main_menu", value)
        self.value_changed.emit({'show_main_menu': value})

    def __getattr__(self, name: str) -> Any:
        """
        Handle attribute access dynamically. Helps to avoid undefined attribute error highlighting within IDE.

        When an AttributeError is raised during attribute access, the __getattr__ method is invoked to handle the
        attribute access dynamically. In your case, after the AttributeError is raised and caught by __getattr__,
        the getter() function is invoked as part of the dynamic attribute access.

        Here's how it works:
        1. The code attempts to access an attribute, such as settings.app_font_size.
        2. If the attribute is not found in the object's namespace, Python raises an AttributeError.
        3. The __getattr__ method intercepts the AttributeError and handles it by providing a custom behavior.
           In this case, it raises another AttributeError with a descriptive message.
        4. After the __getattr__ method completes its execution, the original attribute access attempt is retried.
        5. Since the attribute access is retried, Python again tries to access settings.app_font_size.
        6. This time, the attribute settings.app_font_size exists because it was dynamically created using the
           create_property method.
        7. The getter() function associated with the dynamically created property is invoked to retrieve the value of
           app_font_size
        """
        if name in self.__dict__:
            return self.__dict__[name]
        else:
            raise AttributeError(f"'{type(self).__name__}' object has no attribute '{name}'")

    def clear(self):
        # Clear all entries in the QSettings instance to reset settings
        self.settings.clear()

        # Delete the settings file
        settings_file_path = self.get_filename()
        # Check if file exists and is writable
        if os.path.exists(settings_file_path) and os.access(settings_file_path, os.W_OK):
            try:
                os.remove(settings_file_path)
                return True
            except OSError as e:
                self.logger.warning(f"Error deleting settings file {settings_file_path}: {e}")
                return False
        else:
            self.logger.warning(f"Settings file does not exist or is not writable: {settings_file_path}")
            return False



"""
Notolog Editor
An open-source Markdown editor built with Python.

File Details:
- Purpose: Contains unit and integration tests for the related functionality.

Repository: https://github.com/notolog/notolog-editor
Website: https://notolog.app
PyPI: https://pypi.org/project/notolog

Author: Vadim Bakhrenkov
Copyright: 2024-2025 Vadim Bakhrenkov
License: MIT License

For detailed instructions and project information, please see the repository's README.md.
"""

from notolog.text_block_data import TextBlockData

import pytest


class TestTextBlockData:

    @pytest.fixture(scope="class", autouse=True)
    def test_object(self):
        """
        Fixture to pass storage between the methods below.
        The sequence is matter!
        * https://docs.pytest.org/en/7.4.x/explanation/fixtures.html
        * https://docs.pytest.org/en/latest/example/parametrize.html
        """
        if not hasattr(self, 'test_obj'):
            self.test_obj = TextBlockData(123)
        return self.test_obj

    def test_text_block_data_init(self, test_object, exp_block_number=123):
        assert test_object is not None
        assert test_object.block_number == exp_block_number
        assert isinstance(test_object.data, dict)
        assert test_object.data == {}

    @pytest.mark.parametrize(
        "tag, opened, within, closed, start, end, exp_cnt, exp_one_param, exp_one_val",
        [
            ('s', False, False, False, 1, 9, 1, 'end', 9),
            ('s', True, False, False, 19, 99, 2, 'end', 9),
            ('s', False, True, False, 79, 149, 3, 'start', 1),
            ('some_tag1', False, False, False, 11, 22, 1, 'start', 11),
            ('s', False, False, True, 129, 199, 4, 'end', 9),
            ('some_tag2', False, False, False, 123, 456, 1, 'start', 123),
        ]
    )
    def test_text_block_data_setter(self, test_object,
                                    tag, opened, within, closed, start, end, exp_cnt, exp_one_param, exp_one_val):
        # Put the data to the fixture object
        test_object.put(tag=tag, opened=opened, within=within, closed=closed, start=start, end=end)

        assert tag in test_object.data
        assert len(test_object.data[tag]) == exp_cnt
        assert test_object.get_one(tag)[exp_one_param] == exp_one_val

    @pytest.mark.parametrize("tag, exp_cnt", [('s', 4)])
    def test_text_block_data_get_all(self, test_object, tag, exp_cnt):
        assert tag in test_object.data
        assert len(test_object.data[tag]) == exp_cnt

        data = test_object.get_all(tag)
        assert len(data) == exp_cnt

    @pytest.mark.parametrize("tag, index, exp_start, exp_end", [('s', 2, 79, 149)])
    def test_text_block_data_get_one(self, test_object, tag, index, exp_start, exp_end):
        assert tag in test_object.data

        data = test_object.get_one(tag, index)
        assert data['start'] == exp_start
        assert data['end'] == exp_end

    @pytest.mark.parametrize("tag, param, exp_value", [('s', 'end', 9)])
    def test_text_block_data_get_param(self, test_object, tag, param, exp_value):
        assert tag in test_object.data

        param_value = test_object.get_param(tag, param)
        assert param_value == exp_value

    @pytest.mark.parametrize("tag, search_param, search_value, exp_param, exp_value",
                             [('s', 'start', 79, 'end', 149), ('s', 'end', 99, 'start', 19)])
    def test_text_block_data_search(self, test_object, tag, search_param, search_value, exp_param, exp_value):
        assert tag in test_object.data

        data = test_object.search(tag, search_param, search_value)
        assert data[search_param] == search_value
        assert data[exp_param] == exp_value

    @pytest.mark.parametrize("tag_to_drop, tag_to_keep", [('some_tag1', 'some_tag2'), ('some_tag2', 's')])
    def test_text_block_data_drop(self, test_object, tag_to_drop, tag_to_keep):
        assert tag_to_drop in test_object.data
        assert tag_to_keep in test_object.data

        data = test_object.drop(tag_to_drop)
        assert data is None
        assert tag_to_drop not in test_object.data
        assert tag_to_keep in test_object.data

    @pytest.mark.parametrize("tag, index, check_param, exp_end_before, exp_end_after, exp_cnt_before, exp_cnt_after",
                             [('s', 2, 'end', 149, 199, 4, 3),
                              ('s', 2, 'end', 199, None, 3, 2),
                              ('s', 1, 'end', 99, None, 2, 1),
                              ('s', 0, 'end', 9, None, 1, 0)])
    def test_text_block_data_drop_index(self, test_object,
                                        tag, index, check_param,
                                        exp_end_before, exp_end_after, exp_cnt_before, exp_cnt_after):
        assert tag in test_object.data

        data = test_object.get_one(tag, index)
        assert data is not None
        assert data[check_param] == exp_end_before
        assert len(test_object.data[tag]) == exp_cnt_before

        test_object.drop_index(tag, index)

        data = test_object.get_one(tag, index)
        # Below element has shifted up
        if exp_end_after:
            assert data is not None
            assert data[check_param] == exp_end_after
        else:
            # The last element deleted and nothing expected after it
            assert not data

        assert len(test_object.data[tag]) == exp_cnt_after



"""
Notolog Editor
An open-source Markdown editor built with Python.

File Details:
- Purpose: Contains unit and integration tests for the related functionality.

Repository: https://github.com/notolog/notolog-editor
Website: https://notolog.app
PyPI: https://pypi.org/project/notolog

Author: Vadim Bakhrenkov
Copyright: 2024-2025 Vadim Bakhrenkov
License: MIT License

For detailed instructions and project information, please see the repository's README.md.
"""

import pytest

from unittest.mock import MagicMock, patch
from notolog.font_loader import FontLoader
from notolog.enums.fonts import Fonts

from PySide6.QtGui import QFont


class TestFontLoader:

    @pytest.fixture(autouse=True)
    def mock_logger(self, mocker):
        # Mock the logger for the FontLoader class.
        return mocker.patch("notolog.font_loader.FontLoader.logger")

    @pytest.mark.parametrize(
        "font_enum, font_id, font_family, expected_exception",
        [
            (Fonts.NOTO_SANS, 1, "Noto Sans", None),
            (Fonts.IBM_PLEX_MONO, -1, None, RuntimeError),
        ],
    )
    def test_load_font(self, mocker, font_enum, font_id, font_family, expected_exception):
        # Avoid mocking QFontDatabase directly
        mocker.patch.object(FontLoader, 'load_font', side_effect=lambda font_enum: font_enum.name)

        font_family = FontLoader.load_font(font_enum)
        assert font_family == font_enum.name

    @pytest.mark.parametrize(
        "font_name, font_enum, font_family, expected_exception",
        [
            ("Noto Sans", Fonts.NOTO_SANS, "Noto Sans", None),
            ("Nonexistent Font", None, None, ValueError),
        ],
    )
    def test_get_font_by_name(self, mocker, font_name, font_enum, font_family, expected_exception):
        # Test the get_font_by_name method.
        mocker.patch.object(Fonts, "get_by_name", return_value=font_enum)
        mocker.patch.object(FontLoader, "load_font", return_value=font_family)

        if expected_exception:
            with pytest.raises(expected_exception):
                FontLoader.get_font_by_name(font_name)
        else:
            result = FontLoader.get_font_by_name(font_name)
            assert result == font_family

    def test_get_all_monospace_fonts(self, mocker):
        # Test the get_all_monospace_fonts method.
        monospace_fonts = [Fonts.IBM_PLEX_MONO]
        mocker.patch.object(Fonts, "get_all_monospace", return_value=monospace_fonts)
        mocker.patch.object(FontLoader, "load_font", side_effect=lambda font: font.name)

        result = FontLoader.get_all_monospace_fonts()
        assert result == [font.name for font in monospace_fonts]

    def test_get_monospace_font(self, mocker):
        # Test the get_monospace_font method.
        monospace_fonts = ['IBM Plex Mono']
        mocker.patch.object(FontLoader, "get_all_monospace_fonts", return_value=monospace_fonts)

        result = FontLoader.get_monospace_font()
        assert result == monospace_fonts[0]

    def test_init_fonts(self, mocker):
        # Test the init_fonts method.
        mock_app = MagicMock()
        mock_load_font = mocker.patch.object(FontLoader, "load_font", side_effect=lambda font: font.name)

        with patch.object(Fonts, "__iter__", return_value=iter(Fonts)):
            FontLoader.init_fonts(mock_app)

            for font in Fonts:
                mock_load_font.assert_any_call(font)

                if font.is_default:
                    mock_app.setFont.assert_called_with(QFont(font.name))



# Dutch lexemes settings_dialog.py
lexemes = {
    "tab_module_llama_cpp_config": "Module llama.cpp",

    "module_llama_cpp_config_label": "Module llama.cpp",
    "module_llama_cpp_config_path_label": "Modellocatie",
    "module_llama_cpp_config_path_input_placeholder_text": "Selecteer of voer het modelpad in",
    "module_llama_cpp_config_path_input_accessible_description":
        "Een invoerveld met een selector om het pad van het lokale model op te geven.\n"
        "Ondersteunt modellen in GGUF-formaat, een binair bestandsformaat geoptimaliseerd\n"
        "voor het opslaan van modellen gebruikt met GGML en GGML-gebaseerde uitvoerders.",
    "module_llama_cpp_config_path_input_filter_text": "GGUF-bestanden",

    "module_llama_cpp_config_context_window_label": "Grootte van het contextvenster",
    "module_llama_cpp_config_context_window_input_accessible_description":
        "Stelt het aantal tokens in dat het model overweegt bij het genereren van reacties.\n"
        "Beheert hoeveel voorgaande context wordt gebruikt.",

    "module_llama_cpp_chat_formats_label": "Chatformaten",
    "module_llama_cpp_chat_formats_combo_placeholder_text": "Selecteer een chatformaat",
    "module_llama_cpp_chat_formats_combo_accessible_description":
        "Dropdownmenu om het formaat te selecteren dat gebruikt wordt voor modelgesprekken.",

    "module_llama_cpp_config_system_prompt_label": "Systeemprompt",
    "module_llama_cpp_config_system_prompt_edit_placeholder_text": "Voer de tekst voor de systeemprompt in",
    "module_llama_cpp_config_system_prompt_edit_accessible_description":
        "Tekstveld voor het invoeren van systeemprompts die modelreacties sturen.",

    "module_llama_cpp_config_response_temperature_label": "Respons Temperatuur: {temperature}",
    "module_llama_cpp_config_response_temperature_input_accessible_description":
        "Past de willekeurigheid van de modelreacties aan. Hogere waarden produceren gevarieerdere outputs,\n"
        "terwijl lagere waarden leiden tot voorspelbaardere reacties.",

    "module_llama_cpp_config_response_max_tokens_label": "Maximale Tokens per Reactie",
    "module_llama_cpp_config_response_max_tokens_input_accessible_description":
        "Beperkt het aantal tokens in de reacties van het model tot aan de daadwerkelijke limiet van het contextvenster.\n"
        "Een waarde van nul neemt de capaciteit van het contextvenster aan.",

    "module_llama_cpp_config_prompt_history_size_label": "Grootte van de Promptgeschiedenis",
    "module_llama_cpp_config_prompt_history_size_input_accessible_description":
        "Beheert het aantal invoeringen in de promptgeschiedenis die door het systeem worden bewaard voor referentie.\n"
        "Een waarde van nul staat een onbeperkt aantal invoeringen toe."
}



# Korean lexemes settings_dialog.py
lexemes = {
    # 설정
    "window_title": "설정",

    "button_close": "닫기",

    "tab_general": "일반",
    "tab_editor_config": "편집기",
    "tab_viewer_config": "뷰어",
    "tab_ai_config": "AI 설정",

    "general_app_config_label": "앱 설정",
    "general_app_language_label": "언어",
    "general_app_language_combo_placeholder_text": "언어 선택",
    "general_app_language_combo_accessible_description": "앱의 인터페이스 언어",
    "general_app_theme_label": "테마",
    "general_app_theme_combo_placeholder_text": "테마 선택",
    "general_app_theme_combo_accessible_description": "앱의 인터페이스 테마",
    "general_app_main_menu_label": "메인 메뉴",
    "general_app_main_menu_checkbox": "메인 메뉴 보기",
    "general_app_main_menu_checkbox_accessible_description": "앱의 주요 드롭다운 메뉴 표시",
    "general_app_font_size_label": "글꼴 크기: {size}pt",
    "general_app_font_size_slider_accessible_description": "앱의 전역 글꼴 크기 조정",

    "general_statusbar_label": "상태 바",
    "general_statusbar_show_global_cursor_position_checkbox": "전역 커서 위치 보기",
    "general_statusbar_show_global_cursor_position_checkbox_accessible_description":
        "상태 바에서 전역 커서 위치 표시",

    "editor_config_label": "편집기 설정",
    "editor_config_show_line_numbers_checkbox": "줄 번호 표시",
    "editor_config_show_line_numbers_checkbox_accessible_description": "편집기에서 줄 번호 표시",

    "viewer_config_label": "뷰어 설정",
    "viewer_config_process_emojis_checkbox": "텍스트 이모지를 그래픽으로 변환",
    "viewer_config_process_emojis_checkbox_accessible_description": "텍스트 이모지를 그래픽 표현으로 변환",
    "viewer_config_highlight_todos_checkbox": "할 일 하이라이트",
    "viewer_config_highlight_todos_checkbox_accessible_description": "텍스트 내 할 일 태그 강조",
    "viewer_config_open_link_confirmation_checkbox": "링크 열기 전 확인 요구",
    "viewer_config_open_link_confirmation_checkbox_accessible_description":
        "링크를 열기 전에 확인 요청",
    "viewer_config_save_resources_checkbox": "외부 이미지를 디스크에 자동 저장",
    "viewer_config_save_resources_checkbox_accessible_description":
        "오프라인 액세스를 위해 외부 이미지의 복사본을 디스크에 자동으로 저장합니다.",

    "ai_config_inference_module_label": "추론 모듈",
    "ai_config_inference_module_names_combo_label": "활성 추론 모듈",
    "ai_config_inference_module_names_combo_placeholder_text": "모듈 선택",
    "ai_config_inference_module_names_combo_accessible_description":
        "AI 어시스턴트와 함께 작동할 수 있는 사용 가능한 AI 추론 모듈을 선택하세요.\n"
        "옵션에는 실시간 처리가 가능한 대형 언어 모델(LLM) 또는 API 기반 기능이 포함됩니다.",

    "ai_config_base_label": "기본 매개변수",
    "ai_config_multi_turn_dialogue_checkbox": "대화 메모리를 가진 다중 턴 대화",
    "ai_config_multi_turn_dialogue_checkbox_accessible_description":
        "대화 메모리를 유지하는 다중 턴 대화를 활성화합니다.\n"
        "비활성화되면 새 메시지와 시스템 프롬프트만 응답에 영향을 미칩니다.",
    "ai_config_convert_to_md_checkbox": "결과를 Markdown으로 변환",
    "ai_config_convert_to_md_checkbox_accessible_description":
        "출력 메시지를 Markdown 형식으로 변환합니다.",
}



"""
Notolog Editor
An open-source Markdown editor built with Python.

File Details:
- Purpose: Contains unit and integration tests for the related functionality.

Repository: https://github.com/notolog/notolog-editor
Website: https://notolog.app
PyPI: https://pypi.org/project/notolog

Author: Vadim Bakhrenkov
Copyright: 2024-2025 Vadim Bakhrenkov
License: MIT License

For detailed instructions and project information, please see the repository's README.md.
"""

from notolog.enums.enum_base import EnumBase

assert hasattr(EnumBase, 'default'), "Check default() method is implemented in the base class"


class SomethingEnum(EnumBase):
    """
    Enum for testing 'something' behavior or options.
    """

    SMTH1 = "Smth1"
    SMTH2 = ("Smth2", True)  # Fallback default option
    SMTH3 = "Smth3"



# Georgian lexemes color_picker_dialog.py
lexemes = {
    "color_picker_title": "ფერის არჩევა",

    "color_picker_color_black": "შავი",
    "color_picker_color_silver": "ვერცხლისფერი",
    "color_picker_color_gray": "ნაცრისფერი",
    "color_picker_color_white": "თეთრი",
    "color_picker_color_red": "წითელი",
    "color_picker_color_purple": "იისფერი",
    "color_picker_color_fuchsia": "ფუქსია",
    "color_picker_color_green": "მწვანე",
    "color_picker_color_lime": "ლაიმი",
    "color_picker_color_olive": "ზეთისფერი",
    "color_picker_color_yellow": "ყვითელი",
    "color_picker_color_navy": "ზღვისფერი",
    "color_picker_color_blue": "ლურჯი",
    "color_picker_color_teal": "ტეალი",
    "color_picker_color_aqua": "აკვა",
    "color_picker_color_aliceblue": "ალისის ლურჯი",
    "color_picker_color_antiquewhite": "ანტიკური თეთრი",
    "color_picker_color_aquamarine": "აკვამარინი",
    "color_picker_color_azure": "ცისფერი",
    "color_picker_color_beige": "ბეჟი",
    "color_picker_color_bisque": "ბისკვიტისფერი",
    "color_picker_color_blanchedalmond": "მუქი ნიგოზისფერი",
    "color_picker_color_blueviolet": "ლურჯი იისფერი",
    "color_picker_color_brown": "ყავისფერი",
    "color_picker_color_burlywood": "ბალახისფერი",
    "color_picker_color_cadetblue": "კადეტის ლურჯი",
    "color_picker_color_chartreuse": "ჩარტრიუზი",
    "color_picker_color_chocolate": "შოკოლადისფერი",
    "color_picker_color_coral": "კორალისფერი",
    "color_picker_color_cornflowerblue": "საზამთრო ლურჯი",
    "color_picker_color_cornsilk": "კალათისფერი",
    "color_picker_color_crimson": "წითელი",
    "color_picker_color_cyan": "ციანი",
    "color_picker_color_darkblue": "მუქი ლურჯი",
    "color_picker_color_darkcyan": "მუქი ციანი",
    "color_picker_color_darkgoldenrod": "მუქი ოქროსფერი",
    "color_picker_color_darkgray": "მუქი ნაცრისფერი",
    "color_picker_color_darkgreen": "მუქი მწვანე",
    "color_picker_color_darkkhaki": "მუქი ხაკი",
    "color_picker_color_darkmagenta": "მუქი მაგენტა",
    "color_picker_color_darkolivegreen": "მუქი ზეთისფერი",
    "color_picker_color_darkorange": "მუქი ორანჟი",
    "color_picker_color_darkorchid": "მუქი ორქიდეა",
    "color_picker_color_darkred": "მუქი წითელი",
    "color_picker_color_darksalmon": "მუქი ლოსოსფერი",
    "color_picker_color_darkseagreen": "მუქი ზღვის მწვანე",
    "color_picker_color_darkslateblue": "მუქი დაფნის ლურჯი",
    "color_picker_color_darkslategray": "მუქი დაფნის ნაცრისფერი",
    "color_picker_color_darkturquoise": "მუქი ტურქული ლურჯი",
    "color_picker_color_darkviolet": "მუქი იისფერი",
    "color_picker_color_deeppink": "ღრმა ვარდისფერი",
    "color_picker_color_deepskyblue": "ღრმა ცის ლურჯი",
    "color_picker_color_dimgray": "ბნელი ნაცრისფერი",
    "color_picker_color_dodgerblue": "დოჯერის ლურჯი",
    "color_picker_color_firebrick": "ცეცხლოვანი ყავისფერი",
    "color_picker_color_floralwhite": "ფლორალური თეთრი",
    "color_picker_color_forestgreen": "ტყის მწვანე",
    "color_picker_color_gainsboro": "გეინსბოროს ფერი",
    "color_picker_color_ghostwhite": "მკაცრი თეთრი",
    "color_picker_color_gold": "ოქროსფერი",
    "color_picker_color_goldenrod": "ოქროს ქვეითი",
    "color_picker_color_greenyellow": "მწვანე-ყვითელი",
    "color_picker_color_honeydew": "თაფლის წვეთი",
    "color_picker_color_hotpink": "ცხელი ვარდისფერი",
    "color_picker_color_indianred": "ინდური წითელი",
    "color_picker_color_indigo": "ინდიგო",
    "color_picker_color_ivory": "ზურმუხტისფერი",
    "color_picker_color_khaki": "ხაკი",
    "color_picker_color_lavender": "ლავანდა",
    "color_picker_color_lavenderblush": "ლავანდას ფერი",
    "color_picker_color_lawngreen": "მწვანე ბალახი",
    "color_picker_color_lemonchiffon": "ლიმონის ჩიფონი",
    "color_picker_color_lightblue": "ღია ლურჯი",
    "color_picker_color_lightcoral": "ღია კორალისფერი",
    "color_picker_color_lightcyan": "ღია ციანი",
    "color_picker_color_lightgoldenrodyellow": "ღია ოქროს ფერი",
    "color_picker_color_lightgray": "ღია ნაცრისფერი",
    "color_picker_color_lightgreen": "ღია მწვანე",
    "color_picker_color_lightpink": "ღია ვარდისფერი",
    "color_picker_color_lightsalmon": "ღია ლოსოსფერი",
    "color_picker_color_lightseagreen": "ღია ზღვის მწვანე",
    "color_picker_color_lightskyblue": "ღია ცის ლურჯი",
    "color_picker_color_lightslategray": "ღია დაფნის ნაცრისფერი",
    "color_picker_color_lightsteelblue": "ღია ფოლადის ლურჯი",
    "color_picker_color_lightyellow": "ღია ყვითელი",
    "color_picker_color_limegreen": "ლაიმის მწვანე",
    "color_picker_color_linen": "ბეწვისფერი",
    "color_picker_color_magenta": "მაგენტა",
    "color_picker_color_maroon": "მუქი წითელი",
    "color_picker_color_mediumaquamarine": "საშუალო აკვამარინი",
    "color_picker_color_mediumblue": "საშუალო ლურჯი",
    "color_picker_color_mediumorchid": "საშუალო ორქიდეა",
    "color_picker_color_mediumpurple": "საშუალო იისფერი",
    "color_picker_color_mediumseagreen": "საშუალო ზღვის მწვანე",
    "color_picker_color_mediumslateblue": "საშუალო დაფნის ლურჯი",
    "color_picker_color_mediumspringgreen": "საშუალო გაზაფხულის მწვანე",
    "color_picker_color_mediumturquoise": "საშუალო ტურქული ლურჯი",
    "color_picker_color_mediumvioletred": "საშუალო ვარდისფერი",
    "color_picker_color_midnightblue": "შუაღამის ლურჯი",
    "color_picker_color_mintcream": "მინტის კრემი",
    "color_picker_color_mistyrose": "ნაზი ვარდისფერი",
    "color_picker_color_moccasin": "მოკასინისფერი",
    "color_picker_color_navajowhite": "ნავახოს თეთრი",
    "color_picker_color_oldlace": "ძველი ბეწვი",
    "color_picker_color_olivedrab": "ზეთისფერი",
    "color_picker_color_orange": "ორანჟი",
    "color_picker_color_orangered": "მუქი ორანჟი",
    "color_picker_color_orchid": "ორქიდეა",
    "color_picker_color_palegoldenrod": "ღია ოქროსფერი",
    "color_picker_color_palegreen": "ღია მწვანე",
    "color_picker_color_paleturquoise": "ღია ტურქული ლურჯი",
    "color_picker_color_palevioletred": "ღია ვარდისფერი",
    "color_picker_color_papayawhip": "პაპაიის კრემი",
    "color_picker_color_peachpuff": "ატამის კრემი",
    "color_picker_color_peru": "პერუს ფერი",
    "color_picker_color_pink": "ვარდისფერი",
    "color_picker_color_plum": "ქლიავი",
    "color_picker_color_powderblue": "ფხვნილის ლურჯი",
    "color_picker_color_rebeccapurple": "რებეკას იისფერი",
    "color_picker_color_rosybrown": "როზის ყავისფერი",
    "color_picker_color_royalblue": "მეფის ლურჯი",
    "color_picker_color_saddlebrown": "საყელოს ყავისფერი",
    "color_picker_color_salmon": "ლოსოსფერი",
    "color_picker_color_sandybrown": "ქვიშის ყავისფერი",
    "color_picker_color_seagreen": "ზღვის მწვანე",
    "color_picker_color_seashell": "ზღვის ბალახი",
    "color_picker_color_sienna": "სიენის ფერი",
    "color_picker_color_skyblue": "ცის ლურჯი",
    "color_picker_color_slateblue": "დაფნის ლურჯი",
    "color_picker_color_slategray": "დაფნის ნაცრისფერი",
    "color_picker_color_snow": "თოვლისფერი",
    "color_picker_color_springgreen": "გაზაფხულის მწვანე",
    "color_picker_color_steelblue": "ფოლადის ლურჯი",
    "color_picker_color_tan": "შამფურინისფერი",
    "color_picker_color_thistle": "ბალის ფერი",
    "color_picker_color_tomato": "პომიდორისფერი",
    "color_picker_color_turquoise": "ტურქული ლურჯი",
    "color_picker_color_violet": "იისფერი",
    "color_picker_color_wheat": "ხორბლისფერი",
    "color_picker_color_whitesmoke": "თეთრი კვამლი",
    "color_picker_color_yellowgreen": "ყვითელი-მწვანე",
}



"""
Notolog Editor
An open-source Markdown editor built with Python.

File Details:
- Purpose: Document view decorator to support additional highlights in the resulting text.

Repository: https://github.com/notolog/notolog-editor
Website: https://notolog.app
PyPI: https://pypi.org/project/notolog

Author: Vadim Bakhrenkov
Copyright: 2024-2025 Vadim Bakhrenkov
License: MIT License

For detailed instructions and project information, please see the repository's README.md.
"""

from PySide6.QtGui import QTextCursor, QSyntaxHighlighter

import logging

from typing import TYPE_CHECKING, Union

from .settings import Settings
from .highlight.view_highlighter import ViewHighlighter

if TYPE_CHECKING:
    from PySide6.QtCore import QObject  # noqa: F401
    from PySide6.QtGui import QTextBlockUserData  # noqa: F401
    from PySide6.QtWidgets import QTextBrowser  # noqa: F401
    from .text_block_data import TextBlockData  # noqa: F401


class ViewDecorator:
    """
    View mode result decoration by modifying loaded QTextDocument.
    The source of the data comes from the userData() attached to a block within ViewHighlighter.
    """

    """
    Walking rules to create QRegExp for each pattern
    * tag
    * open index shifting right
    * close index shifting left
    * theme style format key from the highlighter
    * replacement
    """
    rules = [
        # strikethrough
        ('s', 2, 2, 's', None),
        ('s_open', 2, 0, 's', None),
        ('s_close', 0, 2, 's', None),
        ('s_within', 0, 0, 's', None),
        # todos highlighting
        ('todo', 0, 0, 'todo', None),
        # invisible separator
        ('inv_sep', 0, 0, 'inv_sep', None),
    ]

    def __init__(self, highlighter: Union[QSyntaxHighlighter, ViewHighlighter]):
        """
        Args:
            highlighter (Union[QSyntaxHighlighter, ViewHighlighter]):
            Highlighter that holds document to apply any modifications.
        """
        self.highlighter = highlighter
        self.doc = self.highlighter.document()

        self.settings = Settings()

        self.logger = logging.getLogger('view_decorator')

        self.logger.debug('Characters count %d' % self.doc.characterCount())

        cursor = QTextCursor(self.doc)
        self.cursor_pos_orig = cursor.position()

    def restore_cursor_pos(self):
        cursor = QTextCursor(self.doc)
        # Restore original cursor position
        cursor.setPosition(self.cursor_pos_orig, QTextCursor.MoveMode.MoveAnchor)
        parent = self.doc.parent()  # type: QObject
        parent_widget = parent.get_view_widget()  # type: QTextBrowser
        parent_widget.setTextCursor(cursor)

    def process(self):
        cursor = QTextCursor(self.doc)
        cursor.movePosition(QTextCursor.MoveOperation.Start)
        while not (cursor.atEnd()
                   # Sometimes, cursor.atEnd() doesn't work as the cursor unable to move next block at the very end
                   or (cursor.block().blockNumber() > 0 and cursor.block().blockNumber() == self.doc.blockCount() - 1)):
            current_block = cursor.block()
            # current_block_number = current_block.blockNumber()
            block_pos = current_block.position()

            for tag, oi, ci, fmt_key, repl in self.rules:
                if tag == 'todo' and not self.settings.viewer_highlight_todos:
                    continue
                if repl is None:
                    repl = ""
                # Get theme's style format from the highlighter to keep single config and maintain overrides.
                fmt = self.highlighter.theme[fmt_key]
                data_storage = current_block.userData()  # type: Union[QTextBlockUserData, TextBlockData]
                if (data_storage is not None
                        and hasattr(data_storage, 'block_number')
                        and data_storage.get_one(tag)):

                    # Each text cut reduced by token's length
                    len_reduced = 0

                    for block_data in data_storage.get_all(tag):
                        self.logger.debug(
                            'Current block data [%d]~[%d] "%s", in:%r, opened:%r, closed:%r, start: %d, end: %d'
                            % (current_block.blockNumber(), data_storage.block_number, tag, block_data['within'],
                               block_data['opened'], block_data['closed'], block_data['start'], block_data['end']))

                        _r = {'o': block_pos + block_data['start'],
                              'c': block_pos + block_data['end'],
                              'oi': oi, 'ci': ci, 'fmt': fmt, 'repl': repl}

                        find_cursor = QTextCursor(current_block)

                        """
                        # Proof of concept
                        find_cursor = self.doc.find(pattern, find_cursor)
                        if find_cursor.isNull():
                           continue
                        _r = {'o': find_cursor.selectionStart(), 'c': find_cursor.selectionEnd(),
                              'oi': oi, 'ci': ci, 'fmt': fmt, 'repl': repl}
                        self.logger.debug('Cursor position match block: %d, abs: %d, open: %d, close: %d, text: %s'
                            % (find_cursor.positionInBlock(), find_cursor.position(), find_cursor.selectionStart(),
                               find_cursor.selectionEnd(), find_cursor.selectedText()))
                        """

                        # Edit the block
                        find_cursor.beginEditBlock()

                        # Cursor opening
                        open_pos = _r['o'] - len_reduced
                        if _r['oi'] != 0:
                            # open_pos = int(4 * (i - 1))
                            """
                            More about text cursor anchor and movement
                            * https://doc.qt.io/qt-6/qtextcursor.html#anchor
                            * https://doc.qt.io/qt-6/qtextcursor.html#MoveMode-enum

                            More about text cursor position movement
                            * https://doc.qt.io/qt-6/qtextcursor.html#movePosition
                            * https://doc.qt.io/qt-6/qtextcursor.html#MoveOperation-enum

                            `find_cursor.movePosition(QTextCursor.MoveOperation.Right, QTextCursor.MoveMode.KeepAnchor, 1)`

                            More about text cursor selection
                            * https://doc.qt.io/qt-6/qtextcursor.html#SelectionType-enum

                            `find_cursor.select(QTextCursor.SelectionType.WordUnderCursor)`
                            `find_cursor.removeSelectedText()`
                            """
                            find_cursor.setPosition(open_pos, QTextCursor.MoveMode.MoveAnchor)
                            # KeepAnchor - the cursor selects the text it moves over.
                            # The same effect as Shift key + moving cursor.
                            find_cursor.setPosition(open_pos + _r['oi'], QTextCursor.MoveMode.KeepAnchor)
                            find_cursor.insertText(_r['repl'])
                            len_reduced += _r['oi'] - (len(_r['repl']) if _r['repl'] else 0)
                        # Cursor closing
                        close_pos = _r['c'] - len_reduced
                        if _r['ci'] != 0:
                            # close_pos = int(2 * (i - 1)) - int(2 * i) # works only for open-close pair of tokens
                            find_cursor.setPosition(close_pos - _r['ci'], QTextCursor.MoveMode.MoveAnchor)
                            find_cursor.setPosition(close_pos, QTextCursor.MoveMode.KeepAnchor)
                            find_cursor.insertText(_r['repl'])
                            len_reduced += _r['ci'] - (len(_r['repl']) if _r['repl'] else 0)

                        find_cursor.setPosition(open_pos, QTextCursor.MoveMode.MoveAnchor)
                        find_cursor.setPosition(close_pos - _r['ci'], QTextCursor.MoveMode.KeepAnchor)

                        if _r['fmt'] is not None:
                            cfc = _r['fmt'].copy()
                            find_cursor.mergeCharFormat(self.highlighter.cf(**cfc))

                        find_cursor.endEditBlock()

            # Move cursor at the end of the block first if the next command fails
            cursor.movePosition(QTextCursor.MoveOperation.EndOfBlock)
            # Next block
            cursor.movePosition(QTextCursor.MoveOperation.NextBlock)

        # Restore original cursor position
        self.restore_cursor_pos()



"""
Notolog Editor
An open-source Markdown editor built with Python.

File Details:
- Purpose: Extends AI Assistant Dialog Class by providing custom QTextEdit functionality for the prompt input field.

Repository: https://github.com/notolog/notolog-editor
Website: https://notolog.app
PyPI: https://pypi.org/project/notolog

Author: Vadim Bakhrenkov
Copyright: 2024-2025 Vadim Bakhrenkov
License: MIT License

For detailed instructions and project information, please see the repository's README.md.
"""

from PySide6.QtWidgets import QTextEdit
from PySide6.QtGui import QTextOption
from PySide6.QtCore import Qt

from . import Lexemes

import logging


class AIPromptInput(QTextEdit):
    def __init__(self, send_callback, parent=None):
        super().__init__(parent)

        self.parent = parent

        if self.parent and hasattr(self.parent, 'font'):
            # Apply font from the main window to the element
            self.setFont(self.parent.font())

        # Parent settings
        self.settings = self.parent.settings  # type: ignore

        self.logger = logging.getLogger('ai_prompt_input')

        # Load lexemes for selected language and scope
        self.lexemes = Lexemes(self.settings.app_language, default_scope='ai_assistant')

        # Callback function to handle send request
        self.send_callback = send_callback

        self.init_ui()

    def init_ui(self):
        # Set QTextEdit attributes (e.g., placeholders, font, and interaction flags)
        self.setWordWrapMode(QTextOption.WrapMode.WordWrap)
        self.setFont(self.parent.font())
        self.setFocus()

        # Calculate height: font metrics height * 2 + some padding
        text_height = self.fontMetrics().height()
        self.setFixedHeight(text_height * 3 + 10)  # Adjust 10 for padding

        self.setPlaceholderText(
            self.lexemes.get('dialog_prompt_input_placeholder_text'))
        self.setAccessibleDescription(
            self.lexemes.get('dialog_prompt_input_accessible_description'))

        self.setAlignment(Qt.AlignmentFlag.AlignTop | Qt.AlignmentFlag.AlignLeft)

    def keyPressEvent(self, event):
        # Check for Enter key
        if event.key() == Qt.Key.Key_Return or event.key() == Qt.Key.Key_Enter:
            if event.modifiers() == Qt.KeyboardModifier.ControlModifier:
                # Ctrl + Enter: Add a new line
                self.insertPlainText("\n")
            else:
                # Enter: Send the request
                self.send_callback()
        else:
            # Pass other keys to the base class
            super().keyPressEvent(event)



# Latin lexemes settings_dialog.py
lexemes = {
    # Configurationes
    "window_title": "Configurationes",

    "button_close": "Claudere",

    "tab_general": "Generale",
    "tab_editor_config": "Editor",
    "tab_viewer_config": "Visor",
    "tab_ai_config": "Configuratio AI",

    "general_app_config_label": "Configuratio App",
    "general_app_language_label": "Lingua",
    "general_app_language_combo_placeholder_text": "Elige linguam",
    "general_app_language_combo_accessible_description": "Lingua interfaciei app",
    "general_app_theme_label": "Thema",
    "general_app_theme_combo_placeholder_text": "Elige thema",
    "general_app_theme_combo_accessible_description": "Thema interfaciei app",
    "general_app_main_menu_label": "Praecipuum Menu",
    "general_app_main_menu_checkbox": "Praecipuum Menu Ostende",
    "general_app_main_menu_checkbox_accessible_description": "Praecipuum dropdown menu applicationis ostendere",
    "general_app_font_size_label": "Magnitudo Fontis: {size}pt",
    "general_app_font_size_slider_accessible_description": "Regula magnitudinem fontis globalis applicationis",

    "general_statusbar_label": "Status Barra",
    "general_statusbar_show_global_cursor_position_checkbox": "Ostende Globalem Cursoris Positionem",
    "general_statusbar_show_global_cursor_position_checkbox_accessible_description":
        "Ostende globalem cursoris positionem in status barra",

    "editor_config_label": "Editoris Configuratio",
    "editor_config_show_line_numbers_checkbox": "Ostende Numeros Linearum",
    "editor_config_show_line_numbers_checkbox_accessible_description": "Ostende numeros linearum in editor",

    "viewer_config_label": "Visoris Configuratio",
    "viewer_config_process_emojis_checkbox": "Converte Textum Emojis ad Graphica",
    "viewer_config_process_emojis_checkbox_accessible_description":
        "Converte emojis textuales in representationes graphicas",
    "viewer_config_highlight_todos_checkbox": "Illustra TODOs",
    "viewer_config_highlight_todos_checkbox_accessible_description": "In textu TODOs signa illustra",
    "viewer_config_open_link_confirmation_checkbox": "Postulat Confirmationem ad Aperiendum Nexus",
    "viewer_config_open_link_confirmation_checkbox_accessible_description":
        "Quaere confirmationem ante apertionem nexus",
    "viewer_config_save_resources_checkbox": "Serva imagines externas in disco automatice",
    "viewer_config_save_resources_checkbox_accessible_description":
        "Automatice servat exemplaria imaginum externarum in disco ad usum sine nexu.",

    "ai_config_inference_module_label": "Modulus Inferentiae",
    "ai_config_inference_module_names_combo_label": "Modulus Inferentiae Activus",
    "ai_config_inference_module_names_combo_placeholder_text": "Modulum Elige",
    "ai_config_inference_module_names_combo_accessible_description":
        "Elige ex modulis inferentiae AI praesto ut opera cum Auxiliario AI gerantur.\n"
        "Optiones includunt magnos linguarum modelos (LLM) cum processione temporis realis aut functiones\n"
        "quae sunt basis API.",

    "ai_config_base_label": "Parametri Fundamentales",
    "ai_config_multi_turn_dialogue_checkbox": "Dialogus Multiplex cum Memoria Sermonis",
    "ai_config_multi_turn_dialogue_checkbox_accessible_description":
        "Permitte dialogum multiplex qui ultimam praeceptionem ad memoriam sermonis retinet.\n"
        "Si deactivatum, tantum novum nuntium et praeceptum systematis responsionem afficiunt.",
    "ai_config_convert_to_md_checkbox": "Resultatum in Markdown Convertere",
    "ai_config_convert_to_md_checkbox_accessible_description":
        "Convertere nuntium exitu in formam Markdown.",
}



# Swedish lexemes common.py
lexemes = {
    "app_title": "Notolog Redigerare",
    "app_title_with_sub": "{app_title} - {sub_title}",

    "tree_filter_accessible_desc": "Filterfält för filer",

    "menu_action_copy_file_path": "Kopiera filsökväg",
    "menu_action_rename": "Byt namn",
    "menu_action_delete": "Radera",
    "menu_action_delete_completely": "Radera helt",
    "menu_action_restore": "Återställ",
    "menu_action_create_new_dir": "Skapa en ny katalog",

    "dialog_file_rename_title": "Byt namn på fil",
    "dialog_file_rename_field_label": "Ange nytt filnamn",
    "dialog_file_rename_button_ok": "Byt namn",
    "dialog_file_rename_warning_exists": "En fil med samma namn finns redan",

    "dialog_file_delete_title": "Radera fil",
    "dialog_file_delete_text": "Vill du radera filen \"{file_name}\"?",
    "dialog_file_delete_completely_title": "Radera filen helt",
    "dialog_file_delete_completely_text": "Vill du radera filen \"{file_name}\" helt?",
    "dialog_file_delete_error": "Kan inte radera filen, ett fel inträffade",
    "dialog_file_delete_error_not_found": "Filen hittades inte",

    "dialog_file_restore_title": "Återställ fil",
    "dialog_file_restore_text": "Återställ filen \"{file_name}\"?",
    "dialog_file_restore_error": "Kan inte återställa filen, ett fel inträffade",
    "dialog_file_restore_warning_exists": "En fil med namnet {file_name} finns redan",

    "dialog_create_new_dir_title": "Skapa en ny katalog",
    "dialog_create_new_dir_label": "Nytt katalognamn",
    "dialog_create_new_dir_input_placeholder_text": "Ange katalognamn",
    "dialog_create_new_dir_button_ok": "Skapa",
    "dialog_create_new_dir_button_cancel": "Avbryt",
    "dialog_create_new_dir_warning_empty_title": "Nytt katalognamn fel",
    "dialog_create_new_dir_warning_empty_text": "Katalognamnet kan inte vara tomt",
    "dialog_create_new_dir_warning_too_long_title": "Nytt katalognamn fel",
    "dialog_create_new_dir_warning_too_long_text": "Katalognamnet är för långt; maximalt {symbols} tecken tillåtna!",
    "dialog_create_new_dir_error_existed": "Katalogen finns redan",
    "dialog_create_new_dir_error": "Det går inte att skapa katalogen. Se till att målkatalogen {base_dir} är skrivbar",

    "dialog_message_box_title": "Meddelande",
    "dialog_message_box_button_ok": "Stäng",

    "action_new_file_first_line_template_text": "Nytt dokument",
    "action_open_file_dialog_caption": "Öppna fil",
    "action_save_as_file_dialog_caption": "Spara fil",

    "dialog_save_empty_file_title": "Spara tom fil",
    "dialog_save_empty_file_text": "Tillåt att spara filen med tomt innehåll?",

    "dialog_encrypt_file_title": "Kryptera fil",
    "dialog_encrypt_file_text": "Kryptera filen \"{file_name}\"?",
    "encrypt_file_warning_file_is_already_encrypted": "Filen är redan krypterad!",
    "dialog_encrypt_file_rewrite_existing_title": "Skriv över befintlig fil",
    "dialog_encrypt_file_rewrite_existing_text": "Skriv över den befintliga filen \"{file_path}\"?",

    "dialog_decrypt_file_title": "Dekryptera fil",
    "dialog_decrypt_file_text": "Dekryptera filen \"{file_name}\"?",
    "decrypt_file_warning_file_is_not_encrypted": "Filen är inte krypterad!",
    "dialog_decrypt_file_rewrite_existing_title": "Skriv över befintlig fil",
    "dialog_decrypt_file_rewrite_existing_text": "Skriv över den befintliga filen \"{file_path}\"?",

    "dialog_encrypt_new_password_title": "Nytt lösenord",
    "dialog_encrypt_new_password_label": "Lösenord:",
    "dialog_encrypt_new_password_input_placeholder_text": "Ange nytt lösenord",
    "dialog_encrypt_new_password_hint_label": "Ledtråd:",
    "dialog_encrypt_new_password_hint_label_description": "Ledtråden är inte krypterad och kan läsas från filen!"
                                                          "\nUndvik uppenbara ledtrådar som kan gissas lätt, såsom "
                                                          "födelsedatum."
                                                          "\nFörsök använda en referens som inte lätt kan kopplas till dig.",
    "dialog_encrypt_new_password_hint_input_placeholder_text": "Ange ledtråd (valfritt)",
    "dialog_encrypt_new_password_button_ok": "OK",
    "dialog_encrypt_new_password_button_cancel": "Avbryt",
    "dialog_encrypt_new_password_warning_empty_title": "Varning",
    "dialog_encrypt_new_password_warning_empty_text": "Lösenordsfältet får inte vara tomt!",
    "dialog_encrypt_new_password_warning_too_long_title": "Varning",
    "dialog_encrypt_new_password_warning_too_long_text": "Ledtrådsfältet är för långt, max {symbols} tecken!",

    "dialog_encrypt_password_title": "Ange lösenord",
    "dialog_encrypt_password_label": "Lösenord:",
    "dialog_encrypt_password_input_placeholder_text": "Ange lösenord",
    "dialog_encrypt_password_hint_label": "Ledtråd:",
    "dialog_encrypt_password_button_ok": "OK",
    "dialog_encrypt_password_button_cancel": "Avbryt",

    "dialog_encrypt_password_reset_title": "Återställ krypteringslösenord",
    "dialog_encrypt_password_reset_text": "Är du säker på att du vill återställa det nuvarande krypteringslösenordet?",
    "dialog_encrypt_password_reset_button_cancel": "Avbryt",
    "dialog_encrypt_password_reset_button_yes": "Ja",

    "dialog_open_link_title": "Länk",
    "dialog_open_link_text": "Öppna länken \"{url}\" i en webbläsare?",

    "dialog_reset_settings_title": "Återställ inställningar?",
    "dialog_reset_settings_text":
        "All lagrad data i inställningarna kommer att rensas, och appen kommer att startas om för att tillämpa "
        "ändringarna.",

    "dialog_exit_unsaved_title": "Bekräfta Avslut",
    "dialog_exit_unsaved_text": "Den öppnade filen '{file_name}' kan inte sparas. Fortsätt med att avsluta?",

    "message_app_config_file_access": "Åtkomst nekad när man försöker komma åt appens konfigurationsfil på {file_path}. "
                                      "Ställ in rätt behörigheter för att säkerställa korrekt drift.",

    "field_dir_path_dialog_caption": "Välj Katalog",
    "field_file_path_dialog_caption": "Välj Fil",

    "load_file_encryption_password_mismatch": "Krypteringslösenordet stämmer inte!",
    "load_file_encryption_password_incorrect": "Fel krypteringslösenord!",
    "load_file_none_content_error": "Filen kan inte laddas.",

    "action_new_file_error_occurred": "Kan inte skapa fil; ett fel inträffade."
                                      "\nKontrollera behörigheterna i filsystemet.",
    "save_active_file_error_occurred": "Kan inte spara fil; ett fel inträffade.",

    "expandable_block_default_title": "Mer information...",
    "expandable_block_open_close_tags_mismatch_warning": "Mismatch i <details> block öppna/stänga taggar",

    "dialog_color_picker_color_copied_to_the_clipboard": "Formaterad text har kopierats till urklipp",

    "popup_about_title": "Om programmet",
    "popup_about_app_name_description": "Python Markdown-redigerare",

    "popup_about_version": "Version",
    "popup_about_license": "Licens",
    "popup_about_website": "Webbplats",
    "popup_about_repository": "GitHub",
    "popup_about_pypi": "PyPi",
    "popup_about_date": "Datum",

    "update_helper_new_version_is_available": "En ny version {latest_version} av appen är tillgänglig",
    "update_helper_latest_version_installed": "Den senaste versionen av appen är installerad",

    "network_connection_error_empty": "Kan inte hämta svarsinformation",
    "network_connection_error_connection_or_dns":
        "Värden hittades inte. Det kan vara ett problem med internetanslutningen eller DNS.",
    "network_connection_error_connection_refused":
        "Anslutningen nekades. Servern kan vara nere, eller så kan det finnas nätverksproblem.",
    "network_connection_error_connection_timed_out": "Anslutningen löpte ut i tid. Det kan finnas nätverksproblem.",
    "network_connection_error_connection_404_error":
        "404-fel vid anslutning. Den begärda sidan eller resursen hittades inte.",
    "network_connection_error_generic_with_status_code": "Förfrågan misslyckades med statuskod: {status_code}",
}



"""
Notolog Editor
An open-source Markdown editor built with Python.

File Details:
- Purpose: Helper class for file operations.

Repository: https://github.com/notolog/notolog-editor
Website: https://notolog.app
PyPI: https://pypi.org/project/notolog

Author: Vadim Bakhrenkov
Copyright: 2024-2025 Vadim Bakhrenkov
License: MIT License

For detailed instructions and project information, please see the repository's README.md.
"""

from PySide6.QtCore import QDir

import os
import sys

from typing import Union


def res_path(rel_path):
    """
    Generate an absolute path for resource files. This method accommodates environments
    both during development and after deployment using PyInstaller.

    The function tries to determine the base path set by PyInstaller, which stores it
    in the `_MEIPASS` attribute during the bundled application's runtime. If the application
    is not running as a PyInstaller bundle, it defaults to the current directory's absolute path.

    Args:
        rel_path (str): The relative path to the resource.

    Returns:
        str: The absolute path combined from the base path and the relative path.
    """
    try:
        # PyInstaller creates a temporary folder and store its path in _MEIPASS
        base_path = sys._MEIPASS
    except Exception:
        # If running as a live Python script, not a bundled application
        base_path = os.path.abspath(QDir.currentPath())

    return os.path.join(base_path, rel_path)


def size_f(size: int, suffix: str = "B") -> str:
    """
    Convert a file size to human-readable form.

    Args:
        size (int): File size in bytes.
        suffix (str): Suffix for the size unit. Defaults to 'B' (bytes).

    Returns:
        str: Formatted string representing the file size in a human-readable form.
    """
    units = ["", "Ki", "Mi", "Gi", "Ti", "Pi", "Ei", "Zi"]
    unit_size = 1024.0
    i = 0
    while abs(size) >= unit_size and i < len(units) - 1:
        size /= unit_size
        i += 1
    return f"{round(size, 1)}{units[i]}{suffix}"


def read_file(file_path: str, as_bytearray: bool = False) -> Union[str, bytearray]:
    """
    Read file content from the specified path.

    Args:
        file_path (str): The path to the file.
        as_bytearray (bool): If True, returns the content as a bytearray;
            if False, returns the content as a string. Defaults to False.

    Returns:
        str or bytearray or None: Returns the file content as a string or bytearray depending on the value of `b`.
        Returns None if the file cannot be read.
    """

    """
    # Qt implementation could be like
    file = QFile(file_path)
    if file.open(QIODevice.OpenModeFlag.ReadOnly):
        # file.size()
        return file.readAll()
    """
    # a bytes-like object or string
    mode = 'rb' if as_bytearray else 'r'
    # without 3-char extension: file_path[:-4]
    with (open(file_path, mode) if as_bytearray
          else open(file_path, mode, encoding='utf-8') as file):  # No encoding in bin mode
        return file.read()


def save_file(file_path: str, data: Union[str, bytearray], as_bytearray: bool = False) -> bool:
    """
    Save content to the specified file.

    Args:
        file_path (str): The path to the file where content will be saved.
        data (str or bytearray): The content to save, which can be either a string or a bytearray.
        as_bytearray (bool): Indicates whether the data should be saved as a bytearray.
            If False and data is a bytearray, it will be converted to a string before saving.
            Defaults to False.

    Returns:
        bool: True if the content was successfully saved, False otherwise.
    """

    # Check if the directory has write permissions
    if not os.path.isfile(file_path) and not os.access(os.path.dirname(file_path), os.W_OK):
        return False

    # Check if the file has write permissions
    if os.path.isfile(file_path) and not os.access(file_path, os.W_OK):
        return False

    # a bytes-like object or string
    mode = 'wb' if as_bytearray else 'w'

    # without 3-char extension: file_path[:-4]
    with (open(file_path, mode) if as_bytearray
          else open(file_path, mode, encoding='utf-8') as file):  # No encoding in bin mode
        file.write(data)
        return True


def remove_trailing_numbers(text) -> str:
    """
    Remove trailing numbers from the given text. This is commonly used to strip incremental numeric suffixes from
    file names or extensions.

    Args:
        text (str): The text from which trailing digits will be removed.

    Returns:
        str: The text stripped of any trailing digits, if they exist.
    """
    # Start from the end of the string and find the first non-digit character
    i = len(text)
    while i > 0 and text[i - 1].isdigit():
        i -= 1
    # Return the string up to the first trailing digit
    return text[:i]


def is_writable_path(file_path):
    # Check if file exists and if it's writable
    if os.path.exists(file_path):
        if not os.access(file_path, os.W_OK):
            return False
    else:
        # Check if the directory is writable if the file doesn't exist
        parent_dir = os.path.dirname(file_path)
        if not os.access(parent_dir, os.W_OK):
            return False
    return True



# Turkish lexemes main_menu.py
lexemes = {
    "group_file_label": "Dosya",
    "group_edit_label": "Düzenle",
    "group_tools_label": "Araçlar",
    "group_help_label": "Yardım",

    "actions_file_label_new_document": "Yeni belge",
    "actions_file_accessible_name_new_document": "Yeni belge",
    "actions_file_label_open": "Aç",
    "actions_file_accessible_name_open": "Belgeyi aç",
    "actions_file_label_save": "Kaydet",
    "actions_file_accessible_name_save": "Belgeyi kaydet",
    "actions_file_label_save_as": "Farklı Kaydet",
    "actions_file_accessible_name_save_as": "Belgeyi kopya olarak kaydet",
    "actions_file_label_settings": "Ayarlar",
    "actions_file_accessible_name_settings": "Uygulama ayarları",
    "actions_file_label_reset_settings": "Ayarları sıfırla",
    "actions_file_accessible_name_reset_settings": "Ayarları sıfırla",
    "actions_file_label_exit": "Çıkış",
    "actions_file_accessible_name_exit": "Uygulamadan çık",

    "actions_edit_edit_mode": "Düzenle",
    "actions_edit_accessible_name_edit_mode": "Dosyayı düzenle",
    "actions_edit_source_mode": "Kaynak",
    "actions_edit_accessible_name_source_mode": "Kaynak kodu",
    "actions_edit_bold": "Kalın",
    "actions_edit_accessible_name_bold": "Kalın metin biçimi",
    "actions_edit_italic": "İtalik",
    "actions_edit_accessible_name_italic": "İtalik metin biçimi",
    "actions_edit_underline": "Altı çizili",
    "actions_edit_accessible_name_underline": "Altı çizili metin biçimi",
    "actions_edit_strikethrough": "Üstü çizili",
    "actions_edit_accessible_name_strikethrough": "Üstü çizili metin biçimi",
    "actions_edit_blockquote": "Alıntı",
    "actions_edit_accessible_name_blockquote": "Alıntı metin biçimi",

    "actions_tools_ai_assistant": "AI Asistanı",
    "actions_tools_accessible_name_ai_assistant": "AI Asistanını Başlat",
    "actions_tools_color_picker": "Renk Seçici",
    "actions_tools_accessible_name_color_picker": "Metin rengi seç",

    "actions_help_label_md_syntax": "Markdown Sözdizimi",
    "actions_help_accessible_name_md_syntax": "Markdown Sözdizimi",
    "actions_help_label_check_for_updates": "Güncellemeleri Denetle",
    "actions_help_accessible_name_check_for_updates": "Güncellemeleri denetle",
    "actions_help_label_bug_report": "Hata Raporu",
    "actions_help_accessible_name_bug_report": "Hata raporu gönder",
    "actions_help_label_about": "Hakkında",
    "actions_help_accessible_name_about": "Hakkında",
}
